#!/usr/bin/python
from __future__ import (absolute_import, division, print_function)
# Copyright 2019 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fortios_web_proxy_explicit
short_description: Configure explicit Web proxy settings in Fortinet's FortiOS and FortiGate.
description:
    - This module is able to configure a FortiGate or FortiOS by allowing the
      user to set and modify web_proxy feature and explicit category.
      Examples include all parameters and values need to be adjusted to datasources before usage.
      Tested with FOS v6.0.2
version_added: "2.8"
author:
    - Miguel Angel Munoz (@mamunozgonzalez)
    - Nicolas Thomas (@thomnico)
notes:
    - Requires fortiosapi library developed by Fortinet
    - Run as a local_action in your playbook
requirements:
    - fortiosapi>=0.9.8
options:
    host:
       description:
            - FortiOS or FortiGate ip address.
       required: true
    username:
        description:
            - FortiOS or FortiGate username.
        required: true
    password:
        description:
            - FortiOS or FortiGate password.
        default: ""
    vdom:
        description:
            - Virtual domain, among those defined previously. A vdom is a
              virtual instance of the FortiGate that can be configured and
              used as a different unit.
        default: root
    https:
        description:
            - Indicates if the requests towards FortiGate must use HTTPS
              protocol
        type: bool
        default: true
    web_proxy_explicit:
        description:
            - Configure explicit Web proxy settings.
        default: null
        suboptions:
            ftp-incoming-port:
                description:
                    - Accept incoming FTP-over-HTTP requests on one or more ports (0 - 65535, default = 0; use the same as HTTP).
            ftp-over-http:
                description:
                    - Enable to proxy FTP-over-HTTP sessions sent from a web browser.
                choices:
                    - enable
                    - disable
            http-incoming-port:
                description:
                    - Accept incoming HTTP requests on one or more ports (0 - 65535, default = 8080).
            https-incoming-port:
                description:
                    - Accept incoming HTTPS requests on one or more ports (0 - 65535, default = 0, use the same as HTTP).
            https-replacement-message:
                description:
                    - Enable/disable sending the client a replacement message for HTTPS requests.
                choices:
                    - enable
                    - disable
            incoming-ip:
                description:
                    - Restrict the explicit HTTP proxy to only accept sessions from this IP address. An interface must have this IP address.
            incoming-ip6:
                description:
                    - Restrict the explicit web proxy to only accept sessions from this IPv6 address. An interface must have this IPv6 address.
            ipv6-status:
                description:
                    - Enable/disable allowing an IPv6 web proxy destination in policies and all IPv6 related entries in this command.
                choices:
                    - enable
                    - disable
            message-upon-server-error:
                description:
                    - Enable/disable displaying a replacement message when a server error is detected.
                choices:
                    - enable
                    - disable
            outgoing-ip:
                description:
                    - Outgoing HTTP requests will have this IP address as their source address. An interface must have this IP address.
            outgoing-ip6:
                description:
                    - Outgoing HTTP requests will leave this IPv6. Multiple interfaces can be specified. Interfaces must have these IPv6 addresses.
            pac-file-data:
                description:
                    - PAC file contents enclosed in quotes (maximum of 256K bytes).
            pac-file-name:
                description:
                    - Pac file name.
            pac-file-server-port:
                description:
                    - Port number that PAC traffic from client web browsers uses to connect to the explicit web proxy (0 - 65535, default = 0; use the same as
                       HTTP).
            pac-file-server-status:
                description:
                    - Enable/disable Proxy Auto-Configuration (PAC) for users of this explicit proxy profile.
                choices:
                    - enable
                    - disable
            pac-file-url:
                description:
                    - PAC file access URL.
            pac-policy:
                description:
                    - PAC policies.
                suboptions:
                    comments:
                        description:
                            - Optional comments.
                    dstaddr:
                        description:
                            - Destination address objects.
                        suboptions:
                            name:
                                description:
                                    - Address name. Source firewall.address.name firewall.addrgrp.name.
                                required: true
                    pac-file-data:
                        description:
                            - PAC file contents enclosed in quotes (maximum of 256K bytes).
                    pac-file-name:
                        description:
                            - Pac file name.
                    policyid:
                        description:
                            - Policy ID.
                        required: true
                    srcaddr:
                        description:
                            - Source address objects.
                        suboptions:
                            name:
                                description:
                                    - Address name. Source firewall.address.name firewall.addrgrp.name firewall.proxy-address.name firewall.proxy-addrgrp.name.
                                required: true
                    srcaddr6:
                        description:
                            - Source address6 objects.
                        suboptions:
                            name:
                                description:
                                    - Address name. Source firewall.address6.name firewall.addrgrp6.name.
                                required: true
                    status:
                        description:
                            - Enable/disable policy.
                        choices:
                            - enable
                            - disable
            pref-dns-result:
                description:
                    - Prefer resolving addresses using the configured IPv4 or IPv6 DNS server (default = ipv4).
                choices:
                    - ipv4
                    - ipv6
            realm:
                description:
                    - Authentication realm used to identify the explicit web proxy (maximum of 63 characters).
            sec-default-action:
                description:
                    - Accept or deny explicit web proxy sessions when no web proxy firewall policy exists.
                choices:
                    - accept
                    - deny
            socks:
                description:
                    - Enable/disable the SOCKS proxy.
                choices:
                    - enable
                    - disable
            socks-incoming-port:
                description:
                    - Accept incoming SOCKS proxy requests on one or more ports (0 - 65535, default = 0; use the same as HTTP).
            ssl-algorithm:
                description:
                    - "Relative strength of encryption algorithms accepted in HTTPS deep scan: high, medium, or low."
                choices:
                    - low
            status:
                description:
                    - Enable/disable the explicit Web proxy for HTTP and HTTPS session.
                choices:
                    - enable
                    - disable
            strict-guest:
                description:
                    - Enable/disable strict guest user checking by the explicit web proxy.
                choices:
                    - enable
                    - disable
            trace-auth-no-rsp:
                description:
                    - Enable/disable logging timed-out authentication requests.
                choices:
                    - enable
                    - disable
            unknown-http-version:
                description:
                    - Either reject unknown HTTP traffic as malformed or handle unknown HTTP traffic as best as the proxy server can.
                choices:
                    - reject
                    - best-effort
'''

EXAMPLES = '''
- hosts: localhost
  vars:
   host: "192.168.122.40"
   username: "admin"
   password: ""
   vdom: "root"
  tasks:
  - name: Configure explicit Web proxy settings.
    fortios_web_proxy_explicit:
      host:  "{{ host }}"
      username: "{{ username }}"
      password: "{{ password }}"
      vdom:  "{{ vdom }}"
      https: "False"
      web_proxy_explicit:
        ftp-incoming-port: "<your_own_value>"
        ftp-over-http: "enable"
        http-incoming-port: "<your_own_value>"
        https-incoming-port: "<your_own_value>"
        https-replacement-message: "enable"
        incoming-ip: "<your_own_value>"
        incoming-ip6: "<your_own_value>"
        ipv6-status: "enable"
        message-upon-server-error: "enable"
        outgoing-ip: "<your_own_value>"
        outgoing-ip6: "<your_own_value>"
        pac-file-data: "<your_own_value>"
        pac-file-name: "<your_own_value>"
        pac-file-server-port: "<your_own_value>"
        pac-file-server-status: "enable"
        pac-file-url: "<your_own_value>"
        pac-policy:
         -
            comments: "<your_own_value>"
            dstaddr:
             -
                name: "default_name_22 (source firewall.address.name firewall.addrgrp.name)"
            pac-file-data: "<your_own_value>"
            pac-file-name: "<your_own_value>"
            policyid: "25"
            srcaddr:
             -
                name: "default_name_27 (source firewall.address.name firewall.addrgrp.name firewall.proxy-address.name firewall.proxy-addrgrp.name)"
            srcaddr6:
             -
                name: "default_name_29 (source firewall.address6.name firewall.addrgrp6.name)"
            status: "enable"
        pref-dns-result: "ipv4"
        realm: "<your_own_value>"
        sec-default-action: "accept"
        socks: "enable"
        socks-incoming-port: "<your_own_value>"
        ssl-algorithm: "low"
        status: "enable"
        strict-guest: "enable"
        trace-auth-no-rsp: "enable"
        unknown-http-version: "reject"
'''

RETURN = '''
build:
  description: Build number of the fortigate image
  returned: always
  type: str
  sample: '1547'
http_method:
  description: Last method used to provision the content into FortiGate
  returned: always
  type: str
  sample: 'PUT'
http_status:
  description: Last result given by FortiGate on last operation applied
  returned: always
  type: str
  sample: "200"
mkey:
  description: Master key (id) used in the last call to FortiGate
  returned: success
  type: str
  sample: "id"
name:
  description: Name of the table used to fulfill the request
  returned: always
  type: str
  sample: "urlfilter"
path:
  description: Path of the table used to fulfill the request
  returned: always
  type: str
  sample: "webfilter"
revision:
  description: Internal revision number
  returned: always
  type: str
  sample: "17.0.2.10658"
serial:
  description: Serial number of the unit
  returned: always
  type: str
  sample: "FGVMEVYYQT3AB5352"
status:
  description: Indication of the operation's result
  returned: always
  type: str
  sample: "success"
vdom:
  description: Virtual domain used
  returned: always
  type: str
  sample: "root"
version:
  description: Version of the FortiGate
  returned: always
  type: str
  sample: "v5.6.3"

'''

from ansible.module_utils.basic import AnsibleModule


def login(data, fos):
    host = data['host']
    username = data['username']
    password = data['password']

    fos.debug('on')
    if 'https' in data and not data['https']:
        fos.https('off')
    else:
        fos.https('on')

    fos.login(host, username, password)


def filter_web_proxy_explicit_data(json):
    option_list = ['ftp-incoming-port', 'ftp-over-http', 'http-incoming-port',
                   'https-incoming-port', 'https-replacement-message', 'incoming-ip',
                   'incoming-ip6', 'ipv6-status', 'message-upon-server-error',
                   'outgoing-ip', 'outgoing-ip6', 'pac-file-data',
                   'pac-file-name', 'pac-file-server-port', 'pac-file-server-status',
                   'pac-file-url', 'pac-policy', 'pref-dns-result',
                   'realm', 'sec-default-action', 'socks',
                   'socks-incoming-port', 'ssl-algorithm', 'status',
                   'strict-guest', 'trace-auth-no-rsp', 'unknown-http-version']
    dictionary = {}

    for attribute in option_list:
        if attribute in json and json[attribute] is not None:
            dictionary[attribute] = json[attribute]

    return dictionary


def web_proxy_explicit(data, fos):
    vdom = data['vdom']
    web_proxy_explicit_data = data['web_proxy_explicit']
    filtered_data = filter_web_proxy_explicit_data(web_proxy_explicit_data)

    return fos.set('web-proxy',
                   'explicit',
                   data=filtered_data,
                   vdom=vdom)


def fortios_web_proxy(data, fos):
    login(data, fos)

    if data['web_proxy_explicit']:
        resp = web_proxy_explicit(data, fos)

    fos.logout()
    return not resp['status'] == "success", resp['status'] == "success", resp


def main():
    fields = {
        "host": {"required": True, "type": "str"},
        "username": {"required": True, "type": "str"},
        "password": {"required": False, "type": "str", "no_log": True},
        "vdom": {"required": False, "type": "str", "default": "root"},
        "https": {"required": False, "type": "bool", "default": True},
        "web_proxy_explicit": {
            "required": False, "type": "dict",
            "options": {
                "ftp-incoming-port": {"required": False, "type": "str"},
                "ftp-over-http": {"required": False, "type": "str",
                                  "choices": ["enable", "disable"]},
                "http-incoming-port": {"required": False, "type": "str"},
                "https-incoming-port": {"required": False, "type": "str"},
                "https-replacement-message": {"required": False, "type": "str",
                                              "choices": ["enable", "disable"]},
                "incoming-ip": {"required": False, "type": "str"},
                "incoming-ip6": {"required": False, "type": "str"},
                "ipv6-status": {"required": False, "type": "str",
                                "choices": ["enable", "disable"]},
                "message-upon-server-error": {"required": False, "type": "str",
                                              "choices": ["enable", "disable"]},
                "outgoing-ip": {"required": False, "type": "str"},
                "outgoing-ip6": {"required": False, "type": "str"},
                "pac-file-data": {"required": False, "type": "str"},
                "pac-file-name": {"required": False, "type": "str"},
                "pac-file-server-port": {"required": False, "type": "str"},
                "pac-file-server-status": {"required": False, "type": "str",
                                           "choices": ["enable", "disable"]},
                "pac-file-url": {"required": False, "type": "str"},
                "pac-policy": {"required": False, "type": "list",
                               "options": {
                                   "comments": {"required": False, "type": "str"},
                                   "dstaddr": {"required": False, "type": "list",
                                               "options": {
                                                   "name": {"required": True, "type": "str"}
                                               }},
                                   "pac-file-data": {"required": False, "type": "str"},
                                   "pac-file-name": {"required": False, "type": "str"},
                                   "policyid": {"required": True, "type": "int"},
                                   "srcaddr": {"required": False, "type": "list",
                                               "options": {
                                                   "name": {"required": True, "type": "str"}
                                               }},
                                   "srcaddr6": {"required": False, "type": "list",
                                                "options": {
                                                    "name": {"required": True, "type": "str"}
                                                }},
                                   "status": {"required": False, "type": "str",
                                              "choices": ["enable", "disable"]}
                               }},
                "pref-dns-result": {"required": False, "type": "str",
                                    "choices": ["ipv4", "ipv6"]},
                "realm": {"required": False, "type": "str"},
                "sec-default-action": {"required": False, "type": "str",
                                       "choices": ["accept", "deny"]},
                "socks": {"required": False, "type": "str",
                          "choices": ["enable", "disable"]},
                "socks-incoming-port": {"required": False, "type": "str"},
                "ssl-algorithm": {"required": False, "type": "str",
                                  "choices": ["low"]},
                "status": {"required": False, "type": "str",
                           "choices": ["enable", "disable"]},
                "strict-guest": {"required": False, "type": "str",
                                 "choices": ["enable", "disable"]},
                "trace-auth-no-rsp": {"required": False, "type": "str",
                                      "choices": ["enable", "disable"]},
                "unknown-http-version": {"required": False, "type": "str",
                                         "choices": ["reject", "best-effort"]}

            }
        }
    }

    module = AnsibleModule(argument_spec=fields,
                           supports_check_mode=False)
    try:
        from fortiosapi import FortiOSAPI
    except ImportError:
        module.fail_json(msg="fortiosapi module is required")

    fos = FortiOSAPI()

    is_error, has_changed, result = fortios_web_proxy(module.params, fos)

    if not is_error:
        module.exit_json(changed=has_changed, meta=result)
    else:
        module.fail_json(msg="Error in repo", meta=result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
from __future__ import (absolute_import, division, print_function)
# Copyright 2019 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fortios_wireless_controller_wtp
short_description: Configure Wireless Termination Points (WTPs), that is, FortiAPs or APs to be managed by FortiGate in Fortinet's FortiOS and FortiGate.
description:
    - This module is able to configure a FortiGate or FortiOS by allowing the
      user to set and modify wireless_controller feature and wtp category.
      Examples include all parameters and values need to be adjusted to datasources before usage.
      Tested with FOS v6.0.2
version_added: "2.8"
author:
    - Miguel Angel Munoz (@mamunozgonzalez)
    - Nicolas Thomas (@thomnico)
notes:
    - Requires fortiosapi library developed by Fortinet
    - Run as a local_action in your playbook
requirements:
    - fortiosapi>=0.9.8
options:
    host:
       description:
            - FortiOS or FortiGate ip address.
       required: true
    username:
        description:
            - FortiOS or FortiGate username.
        required: true
    password:
        description:
            - FortiOS or FortiGate password.
        default: ""
    vdom:
        description:
            - Virtual domain, among those defined previously. A vdom is a
              virtual instance of the FortiGate that can be configured and
              used as a different unit.
        default: root
    https:
        description:
            - Indicates if the requests towards FortiGate must use HTTPS
              protocol
        type: bool
        default: true
    wireless_controller_wtp:
        description:
            - Configure Wireless Termination Points (WTPs), that is, FortiAPs or APs to be managed by FortiGate.
        default: null
        suboptions:
            state:
                description:
                    - Indicates whether to create or remove the object
                choices:
                    - present
                    - absent
            admin:
                description:
                    - Configure how the FortiGate operating as a wireless controller discovers and manages this WTP, AP or FortiAP.
                choices:
                    - discovered
                    - disable
                    - enable
            allowaccess:
                description:
                    - Control management access to the managed WTP, FortiAP, or AP. Separate entries with a space.
                choices:
                    - telnet
                    - http
                    - https
                    - ssh
            bonjour-profile:
                description:
                    - Bonjour profile name. Source wireless-controller.bonjour-profile.name.
            coordinate-enable:
                description:
                    - Enable/disable WTP coordinates (X,Y axis).
                choices:
                    - enable
                    - disable
            coordinate-latitude:
                description:
                    - WTP latitude coordinate.
            coordinate-longitude:
                description:
                    - WTP longitude coordinate.
            coordinate-x:
                description:
                    - X axis coordinate.
            coordinate-y:
                description:
                    - Y axis coordinate.
            image-download:
                description:
                    - Enable/disable WTP image download.
                choices:
                    - enable
                    - disable
            index:
                description:
                    - Index (0 - 4294967295).
            ip-fragment-preventing:
                description:
                    - Method by which IP fragmentation is prevented for CAPWAP tunneled control and data packets (default = tcp-mss-adjust).
                choices:
                    - tcp-mss-adjust
                    - icmp-unreachable
            lan:
                description:
                    - WTP LAN port mapping.
                suboptions:
                    port-mode:
                        description:
                            - LAN port mode.
                        choices:
                            - offline
                            - nat-to-wan
                            - bridge-to-wan
                            - bridge-to-ssid
                    port-ssid:
                        description:
                            - Bridge LAN port to SSID. Source wireless-controller.vap.name.
                    port1-mode:
                        description:
                            - LAN port 1 mode.
                        choices:
                            - offline
                            - nat-to-wan
                            - bridge-to-wan
                            - bridge-to-ssid
                    port1-ssid:
                        description:
                            - Bridge LAN port 1 to SSID. Source wireless-controller.vap.name.
                    port2-mode:
                        description:
                            - LAN port 2 mode.
                        choices:
                            - offline
                            - nat-to-wan
                            - bridge-to-wan
                            - bridge-to-ssid
                    port2-ssid:
                        description:
                            - Bridge LAN port 2 to SSID. Source wireless-controller.vap.name.
                    port3-mode:
                        description:
                            - LAN port 3 mode.
                        choices:
                            - offline
                            - nat-to-wan
                            - bridge-to-wan
                            - bridge-to-ssid
                    port3-ssid:
                        description:
                            - Bridge LAN port 3 to SSID. Source wireless-controller.vap.name.
                    port4-mode:
                        description:
                            - LAN port 4 mode.
                        choices:
                            - offline
                            - nat-to-wan
                            - bridge-to-wan
                            - bridge-to-ssid
                    port4-ssid:
                        description:
                            - Bridge LAN port 4 to SSID. Source wireless-controller.vap.name.
                    port5-mode:
                        description:
                            - LAN port 5 mode.
                        choices:
                            - offline
                            - nat-to-wan
                            - bridge-to-wan
                            - bridge-to-ssid
                    port5-ssid:
                        description:
                            - Bridge LAN port 5 to SSID. Source wireless-controller.vap.name.
                    port6-mode:
                        description:
                            - LAN port 6 mode.
                        choices:
                            - offline
                            - nat-to-wan
                            - bridge-to-wan
                            - bridge-to-ssid
                    port6-ssid:
                        description:
                            - Bridge LAN port 6 to SSID. Source wireless-controller.vap.name.
                    port7-mode:
                        description:
                            - LAN port 7 mode.
                        choices:
                            - offline
                            - nat-to-wan
                            - bridge-to-wan
                            - bridge-to-ssid
                    port7-ssid:
                        description:
                            - Bridge LAN port 7 to SSID. Source wireless-controller.vap.name.
                    port8-mode:
                        description:
                            - LAN port 8 mode.
                        choices:
                            - offline
                            - nat-to-wan
                            - bridge-to-wan
                            - bridge-to-ssid
                    port8-ssid:
                        description:
                            - Bridge LAN port 8 to SSID. Source wireless-controller.vap.name.
            led-state:
                description:
                    - Enable to allow the FortiAPs LEDs to light. Disable to keep the LEDs off. You may want to keep the LEDs off so they are not distracting
                       in low light areas etc.
                choices:
                    - enable
                    - disable
            location:
                description:
                    - Field for describing the physical location of the WTP, AP or FortiAP.
            login-passwd:
                description:
                    - Set the managed WTP, FortiAP, or AP's administrator password.
            login-passwd-change:
                description:
                    - Change or reset the administrator password of a managed WTP, FortiAP or AP (yes, default, or no, default = no).
                choices:
                    - yes
                    - default
                    - no
            mesh-bridge-enable:
                description:
                    - Enable/disable mesh Ethernet bridge when WTP is configured as a mesh branch/leaf AP.
                choices:
                    - default
                    - enable
                    - disable
            name:
                description:
                    - WTP, AP or FortiAP configuration name.
            override-allowaccess:
                description:
                    - Enable to override the WTP profile management access configuration.
                choices:
                    - enable
                    - disable
            override-ip-fragment:
                description:
                    - Enable/disable overriding the WTP profile IP fragment prevention setting.
                choices:
                    - enable
                    - disable
            override-lan:
                description:
                    - Enable to override the WTP profile LAN port setting.
                choices:
                    - enable
                    - disable
            override-led-state:
                description:
                    - Enable to override the profile LED state setting for this FortiAP. You must enable this option to use the led-state command to turn off
                       the FortiAP's LEDs.
                choices:
                    - enable
                    - disable
            override-login-passwd-change:
                description:
                    - Enable to override the WTP profile login-password (administrator password) setting.
                choices:
                    - enable
                    - disable
            override-split-tunnel:
                description:
                    - Enable/disable overriding the WTP profile split tunneling setting.
                choices:
                    - enable
                    - disable
            override-wan-port-mode:
                description:
                    - Enable/disable overriding the wan-port-mode in the WTP profile.
                choices:
                    - enable
                    - disable
            radio-1:
                description:
                    - Configuration options for radio 1.
                suboptions:
                    auto-power-high:
                        description:
                            - Automatic transmission power high limit in decibels (dB) of the measured power referenced to one milliwatt (mW), or dBm (10 - 17
                               dBm, default = 17).
                    auto-power-level:
                        description:
                            - Enable/disable automatic power-level adjustment to prevent co-channel interference (default = enable).
                        choices:
                            - enable
                            - disable
                    auto-power-low:
                        description:
                            - Automatic transmission power low limit in dBm (the actual range of transmit power depends on the AP platform type).
                    band:
                        description:
                            - WiFi band that Radio 1 operates on.
                        choices:
                            - 802.11a
                            - 802.11b
                            - 802.11g
                            - 802.11n
                            - 802.11n-5G
                            - 802.11n,g-only
                            - 802.11g-only
                            - 802.11n-only
                            - 802.11n-5G-only
                            - 802.11ac
                            - 802.11ac,n-only
                            - 802.11ac-only
                    channel:
                        description:
                            - Selected list of wireless radio channels.
                        suboptions:
                            chan:
                                description:
                                    - Channel number.
                                required: true
                    override-analysis:
                        description:
                            - Enable to override the WTP profile spectrum analysis configuration.
                        choices:
                            - enable
                            - disable
                    override-band:
                        description:
                            - Enable to override the WTP profile band setting.
                        choices:
                            - enable
                            - disable
                    override-channel:
                        description:
                            - Enable to override WTP profile channel settings.
                        choices:
                            - enable
                            - disable
                    override-txpower:
                        description:
                            - Enable to override the WTP profile power level configuration.
                        choices:
                            - enable
                            - disable
                    override-vaps:
                        description:
                            - Enable to override WTP profile Virtual Access Point (VAP) settings.
                        choices:
                            - enable
                            - disable
                    power-level:
                        description:
                            - Radio power level as a percentage of the maximum transmit power (0 - 100, default = 100).
                    radio-id:
                        description:
                            - radio-id
                    spectrum-analysis:
                        description:
                            - Enable/disable spectrum analysis to find interference that would negatively impact wireless performance.
                        choices:
                            - enable
                            - disable
                    vap-all:
                        description:
                            - Enable/disable the automatic inheritance of all Virtual Access Points (VAPs) (default = enable).
                        choices:
                            - enable
                            - disable
                    vaps:
                        description:
                            - Manually selected list of Virtual Access Points (VAPs).
                        suboptions:
                            name:
                                description:
                                    - Virtual Access Point (VAP) name. Source wireless-controller.vap-group.name wireless-controller.vap.name.
                                required: true
            radio-2:
                description:
                    - Configuration options for radio 2.
                suboptions:
                    auto-power-high:
                        description:
                            - Automatic transmission power high limit in decibels (dB) of the measured power referenced to one milliwatt (mW), or dBm (10 - 17
                               dBm, default = 17).
                    auto-power-level:
                        description:
                            - Enable/disable automatic power-level adjustment to prevent co-channel interference (default = enable).
                        choices:
                            - enable
                            - disable
                    auto-power-low:
                        description:
                            - Automatic transmission power low limit in dBm (the actual range of transmit power depends on the AP platform type).
                    band:
                        description:
                            - WiFi band that Radio 1 operates on.
                        choices:
                            - 802.11a
                            - 802.11b
                            - 802.11g
                            - 802.11n
                            - 802.11n-5G
                            - 802.11n,g-only
                            - 802.11g-only
                            - 802.11n-only
                            - 802.11n-5G-only
                            - 802.11ac
                            - 802.11ac,n-only
                            - 802.11ac-only
                    channel:
                        description:
                            - Selected list of wireless radio channels.
                        suboptions:
                            chan:
                                description:
                                    - Channel number.
                                required: true
                    override-analysis:
                        description:
                            - Enable to override the WTP profile spectrum analysis configuration.
                        choices:
                            - enable
                            - disable
                    override-band:
                        description:
                            - Enable to override the WTP profile band setting.
                        choices:
                            - enable
                            - disable
                    override-channel:
                        description:
                            - Enable to override WTP profile channel settings.
                        choices:
                            - enable
                            - disable
                    override-txpower:
                        description:
                            - Enable to override the WTP profile power level configuration.
                        choices:
                            - enable
                            - disable
                    override-vaps:
                        description:
                            - Enable to override WTP profile Virtual Access Point (VAP) settings.
                        choices:
                            - enable
                            - disable
                    power-level:
                        description:
                            - Radio power level as a percentage of the maximum transmit power (0 - 100, default = 100).
                    radio-id:
                        description:
                            - radio-id
                    spectrum-analysis:
                        description:
                            - Enable/disable spectrum analysis to find interference that would negatively impact wireless performance.
                        choices:
                            - enable
                            - disable
                    vap-all:
                        description:
                            - Enable/disable the automatic inheritance of all Virtual Access Points (VAPs) (default = enable).
                        choices:
                            - enable
                            - disable
                    vaps:
                        description:
                            - Manually selected list of Virtual Access Points (VAPs).
                        suboptions:
                            name:
                                description:
                                    - Virtual Access Point (VAP) name. Source wireless-controller.vap-group.name wireless-controller.vap.name.
                                required: true
            split-tunneling-acl:
                description:
                    - Split tunneling ACL filter list.
                suboptions:
                    dest-ip:
                        description:
                            - Destination IP and mask for the split-tunneling subnet.
                    id:
                        description:
                            - ID.
                        required: true
            split-tunneling-acl-local-ap-subnet:
                description:
                    - Enable/disable automatically adding local subnetwork of FortiAP to split-tunneling ACL (default = disable).
                choices:
                    - enable
                    - disable
            split-tunneling-acl-path:
                description:
                    - Split tunneling ACL path is local/tunnel.
                choices:
                    - tunnel
                    - local
            tun-mtu-downlink:
                description:
                    - Downlink tunnel MTU in octets. Set the value to either 0 (by default), 576, or 1500.
            tun-mtu-uplink:
                description:
                    - Uplink tunnel maximum transmission unit (MTU) in octets (eight-bit bytes). Set the value to either 0 (by default), 576, or 1500.
            wan-port-mode:
                description:
                    - Enable/disable using the FortiAP WAN port as a LAN port.
                choices:
                    - wan-lan
                    - wan-only
            wtp-id:
                description:
                    - WTP ID.
                required: true
            wtp-mode:
                description:
                    - WTP, AP, or FortiAP operating mode; normal (by default) or remote. A tunnel mode SSID can be assigned to an AP in normal mode but not
                       remote mode, while a local-bridge mode SSID can be assigned to an AP in either normal mode or remote mode.
                choices:
                    - normal
                    - remote
            wtp-profile:
                description:
                    - WTP profile name to apply to this WTP, AP or FortiAP. Source wireless-controller.wtp-profile.name.
'''

EXAMPLES = '''
- hosts: localhost
  vars:
   host: "192.168.122.40"
   username: "admin"
   password: ""
   vdom: "root"
  tasks:
  - name: Configure Wireless Termination Points (WTPs), that is, FortiAPs or APs to be managed by FortiGate.
    fortios_wireless_controller_wtp:
      host:  "{{ host }}"
      username: "{{ username }}"
      password: "{{ password }}"
      vdom:  "{{ vdom }}"
      https: "False"
      wireless_controller_wtp:
        state: "present"
        admin: "discovered"
        allowaccess: "telnet"
        bonjour-profile: "<your_own_value> (source wireless-controller.bonjour-profile.name)"
        coordinate-enable: "enable"
        coordinate-latitude: "<your_own_value>"
        coordinate-longitude: "<your_own_value>"
        coordinate-x: "<your_own_value>"
        coordinate-y: "<your_own_value>"
        image-download: "enable"
        index: "12"
        ip-fragment-preventing: "tcp-mss-adjust"
        lan:
            port-mode: "offline"
            port-ssid: "<your_own_value> (source wireless-controller.vap.name)"
            port1-mode: "offline"
            port1-ssid: "<your_own_value> (source wireless-controller.vap.name)"
            port2-mode: "offline"
            port2-ssid: "<your_own_value> (source wireless-controller.vap.name)"
            port3-mode: "offline"
            port3-ssid: "<your_own_value> (source wireless-controller.vap.name)"
            port4-mode: "offline"
            port4-ssid: "<your_own_value> (source wireless-controller.vap.name)"
            port5-mode: "offline"
            port5-ssid: "<your_own_value> (source wireless-controller.vap.name)"
            port6-mode: "offline"
            port6-ssid: "<your_own_value> (source wireless-controller.vap.name)"
            port7-mode: "offline"
            port7-ssid: "<your_own_value> (source wireless-controller.vap.name)"
            port8-mode: "offline"
            port8-ssid: "<your_own_value> (source wireless-controller.vap.name)"
        led-state: "enable"
        location: "<your_own_value>"
        login-passwd: "<your_own_value>"
        login-passwd-change: "yes"
        mesh-bridge-enable: "default"
        name: "default_name_38"
        override-allowaccess: "enable"
        override-ip-fragment: "enable"
        override-lan: "enable"
        override-led-state: "enable"
        override-login-passwd-change: "enable"
        override-split-tunnel: "enable"
        override-wan-port-mode: "enable"
        radio-1:
            auto-power-high: "47"
            auto-power-level: "enable"
            auto-power-low: "49"
            band: "802.11a"
            channel:
             -
                chan: "<your_own_value>"
            override-analysis: "enable"
            override-band: "enable"
            override-channel: "enable"
            override-txpower: "enable"
            override-vaps: "enable"
            power-level: "58"
            radio-id: "59"
            spectrum-analysis: "enable"
            vap-all: "enable"
            vaps:
             -
                name: "default_name_63 (source wireless-controller.vap-group.name wireless-controller.vap.name)"
        radio-2:
            auto-power-high: "65"
            auto-power-level: "enable"
            auto-power-low: "67"
            band: "802.11a"
            channel:
             -
                chan: "<your_own_value>"
            override-analysis: "enable"
            override-band: "enable"
            override-channel: "enable"
            override-txpower: "enable"
            override-vaps: "enable"
            power-level: "76"
            radio-id: "77"
            spectrum-analysis: "enable"
            vap-all: "enable"
            vaps:
             -
                name: "default_name_81 (source wireless-controller.vap-group.name wireless-controller.vap.name)"
        split-tunneling-acl:
         -
            dest-ip: "<your_own_value>"
            id:  "84"
        split-tunneling-acl-local-ap-subnet: "enable"
        split-tunneling-acl-path: "tunnel"
        tun-mtu-downlink: "87"
        tun-mtu-uplink: "88"
        wan-port-mode: "wan-lan"
        wtp-id: "<your_own_value>"
        wtp-mode: "normal"
        wtp-profile: "<your_own_value> (source wireless-controller.wtp-profile.name)"
'''

RETURN = '''
build:
  description: Build number of the fortigate image
  returned: always
  type: str
  sample: '1547'
http_method:
  description: Last method used to provision the content into FortiGate
  returned: always
  type: str
  sample: 'PUT'
http_status:
  description: Last result given by FortiGate on last operation applied
  returned: always
  type: str
  sample: "200"
mkey:
  description: Master key (id) used in the last call to FortiGate
  returned: success
  type: str
  sample: "id"
name:
  description: Name of the table used to fulfill the request
  returned: always
  type: str
  sample: "urlfilter"
path:
  description: Path of the table used to fulfill the request
  returned: always
  type: str
  sample: "webfilter"
revision:
  description: Internal revision number
  returned: always
  type: str
  sample: "17.0.2.10658"
serial:
  description: Serial number of the unit
  returned: always
  type: str
  sample: "FGVMEVYYQT3AB5352"
status:
  description: Indication of the operation's result
  returned: always
  type: str
  sample: "success"
vdom:
  description: Virtual domain used
  returned: always
  type: str
  sample: "root"
version:
  description: Version of the FortiGate
  returned: always
  type: str
  sample: "v5.6.3"

'''

from ansible.module_utils.basic import AnsibleModule


def login(data, fos):
    host = data['host']
    username = data['username']
    password = data['password']

    fos.debug('on')
    if 'https' in data and not data['https']:
        fos.https('off')
    else:
        fos.https('on')

    fos.login(host, username, password)


def filter_wireless_controller_wtp_data(json):
    option_list = ['admin', 'allowaccess', 'bonjour-profile',
                   'coordinate-enable', 'coordinate-latitude', 'coordinate-longitude',
                   'coordinate-x', 'coordinate-y', 'image-download',
                   'index', 'ip-fragment-preventing', 'lan',
                   'led-state', 'location', 'login-passwd',
                   'login-passwd-change', 'mesh-bridge-enable', 'name',
                   'override-allowaccess', 'override-ip-fragment', 'override-lan',
                   'override-led-state', 'override-login-passwd-change', 'override-split-tunnel',
                   'override-wan-port-mode', 'radio-1', 'radio-2',
                   'split-tunneling-acl', 'split-tunneling-acl-local-ap-subnet', 'split-tunneling-acl-path',
                   'tun-mtu-downlink', 'tun-mtu-uplink', 'wan-port-mode',
                   'wtp-id', 'wtp-mode', 'wtp-profile']
    dictionary = {}

    for attribute in option_list:
        if attribute in json and json[attribute] is not None:
            dictionary[attribute] = json[attribute]

    return dictionary


def flatten_multilists_attributes(data):
    multilist_attrs = []

    for attr in multilist_attrs:
        try:
            path = "data['" + "']['".join(elem for elem in attr) + "']"
            current_val = eval(path)
            flattened_val = ' '.join(elem for elem in current_val)
            exec(path + '= flattened_val')
        except BaseException:
            pass

    return data


def wireless_controller_wtp(data, fos):
    vdom = data['vdom']
    wireless_controller_wtp_data = data['wireless_controller_wtp']
    flattened_data = flatten_multilists_attributes(wireless_controller_wtp_data)
    filtered_data = filter_wireless_controller_wtp_data(flattened_data)
    if wireless_controller_wtp_data['state'] == "present":
        return fos.set('wireless-controller',
                       'wtp',
                       data=filtered_data,
                       vdom=vdom)

    elif wireless_controller_wtp_data['state'] == "absent":
        return fos.delete('wireless-controller',
                          'wtp',
                          mkey=filtered_data['wtp-id'],
                          vdom=vdom)


def fortios_wireless_controller(data, fos):
    login(data, fos)

    if data['wireless_controller_wtp']:
        resp = wireless_controller_wtp(data, fos)

    fos.logout()
    return not resp['status'] == "success", resp['status'] == "success", resp


def main():
    fields = {
        "host": {"required": True, "type": "str"},
        "username": {"required": True, "type": "str"},
        "password": {"required": False, "type": "str", "no_log": True},
        "vdom": {"required": False, "type": "str", "default": "root"},
        "https": {"required": False, "type": "bool", "default": True},
        "wireless_controller_wtp": {
            "required": False, "type": "dict",
            "options": {
                "state": {"required": True, "type": "str",
                          "choices": ["present", "absent"]},
                "admin": {"required": False, "type": "str",
                          "choices": ["discovered", "disable", "enable"]},
                "allowaccess": {"required": False, "type": "str",
                                "choices": ["telnet", "http", "https",
                                            "ssh"]},
                "bonjour-profile": {"required": False, "type": "str"},
                "coordinate-enable": {"required": False, "type": "str",
                                      "choices": ["enable", "disable"]},
                "coordinate-latitude": {"required": False, "type": "str"},
                "coordinate-longitude": {"required": False, "type": "str"},
                "coordinate-x": {"required": False, "type": "str"},
                "coordinate-y": {"required": False, "type": "str"},
                "image-download": {"required": False, "type": "str",
                                   "choices": ["enable", "disable"]},
                "index": {"required": False, "type": "int"},
                "ip-fragment-preventing": {"required": False, "type": "str",
                                           "choices": ["tcp-mss-adjust", "icmp-unreachable"]},
                "lan": {"required": False, "type": "dict",
                        "options": {
                            "port-mode": {"required": False, "type": "str",
                                          "choices": ["offline", "nat-to-wan", "bridge-to-wan",
                                                      "bridge-to-ssid"]},
                            "port-ssid": {"required": False, "type": "str"},
                            "port1-mode": {"required": False, "type": "str",
                                           "choices": ["offline", "nat-to-wan", "bridge-to-wan",
                                                       "bridge-to-ssid"]},
                            "port1-ssid": {"required": False, "type": "str"},
                            "port2-mode": {"required": False, "type": "str",
                                           "choices": ["offline", "nat-to-wan", "bridge-to-wan",
                                                       "bridge-to-ssid"]},
                            "port2-ssid": {"required": False, "type": "str"},
                            "port3-mode": {"required": False, "type": "str",
                                           "choices": ["offline", "nat-to-wan", "bridge-to-wan",
                                                       "bridge-to-ssid"]},
                            "port3-ssid": {"required": False, "type": "str"},
                            "port4-mode": {"required": False, "type": "str",
                                           "choices": ["offline", "nat-to-wan", "bridge-to-wan",
                                                       "bridge-to-ssid"]},
                            "port4-ssid": {"required": False, "type": "str"},
                            "port5-mode": {"required": False, "type": "str",
                                           "choices": ["offline", "nat-to-wan", "bridge-to-wan",
                                                       "bridge-to-ssid"]},
                            "port5-ssid": {"required": False, "type": "str"},
                            "port6-mode": {"required": False, "type": "str",
                                           "choices": ["offline", "nat-to-wan", "bridge-to-wan",
                                                       "bridge-to-ssid"]},
                            "port6-ssid": {"required": False, "type": "str"},
                            "port7-mode": {"required": False, "type": "str",
                                           "choices": ["offline", "nat-to-wan", "bridge-to-wan",
                                                       "bridge-to-ssid"]},
                            "port7-ssid": {"required": False, "type": "str"},
                            "port8-mode": {"required": False, "type": "str",
                                           "choices": ["offline", "nat-to-wan", "bridge-to-wan",
                                                       "bridge-to-ssid"]},
                            "port8-ssid": {"required": False, "type": "str"}
                        }},
                "led-state": {"required": False, "type": "str",
                              "choices": ["enable", "disable"]},
                "location": {"required": False, "type": "str"},
                "login-passwd": {"required": False, "type": "str"},
                "login-passwd-change": {"required": False, "type": "str",
                                        "choices": ["yes", "default", "no"]},
                "mesh-bridge-enable": {"required": False, "type": "str",
                                       "choices": ["default", "enable", "disable"]},
                "name": {"required": False, "type": "str"},
                "override-allowaccess": {"required": False, "type": "str",
                                         "choices": ["enable", "disable"]},
                "override-ip-fragment": {"required": False, "type": "str",
                                         "choices": ["enable", "disable"]},
                "override-lan": {"required": False, "type": "str",
                                 "choices": ["enable", "disable"]},
                "override-led-state": {"required": False, "type": "str",
                                       "choices": ["enable", "disable"]},
                "override-login-passwd-change": {"required": False, "type": "str",
                                                 "choices": ["enable", "disable"]},
                "override-split-tunnel": {"required": False, "type": "str",
                                          "choices": ["enable", "disable"]},
                "override-wan-port-mode": {"required": False, "type": "str",
                                           "choices": ["enable", "disable"]},
                "radio-1": {"required": False, "type": "dict",
                            "options": {
                                "auto-power-high": {"required": False, "type": "int"},
                                "auto-power-level": {"required": False, "type": "str",
                                                     "choices": ["enable", "disable"]},
                                "auto-power-low": {"required": False, "type": "int"},
                                "band": {"required": False, "type": "str",
                                         "choices": ["802.11a", "802.11b", "802.11g",
                                                     "802.11n", "802.11n-5G", "802.11n,g-only",
                                                     "802.11g-only", "802.11n-only", "802.11n-5G-only",
                                                     "802.11ac", "802.11ac,n-only", "802.11ac-only"]},
                                "channel": {"required": False, "type": "list",
                                            "options": {
                                                "chan": {"required": True, "type": "str"}
                                            }},
                                "override-analysis": {"required": False, "type": "str",
                                                      "choices": ["enable", "disable"]},
                                "override-band": {"required": False, "type": "str",
                                                  "choices": ["enable", "disable"]},
                                "override-channel": {"required": False, "type": "str",
                                                     "choices": ["enable", "disable"]},
                                "override-txpower": {"required": False, "type": "str",
                                                     "choices": ["enable", "disable"]},
                                "override-vaps": {"required": False, "type": "str",
                                                  "choices": ["enable", "disable"]},
                                "power-level": {"required": False, "type": "int"},
                                "radio-id": {"required": False, "type": "int"},
                                "spectrum-analysis": {"required": False, "type": "str",
                                                      "choices": ["enable", "disable"]},
                                "vap-all": {"required": False, "type": "str",
                                            "choices": ["enable", "disable"]},
                                "vaps": {"required": False, "type": "list",
                                         "options": {
                                             "name": {"required": True, "type": "str"}
                                         }}
                            }},
                "radio-2": {"required": False, "type": "dict",
                            "options": {
                                "auto-power-high": {"required": False, "type": "int"},
                                "auto-power-level": {"required": False, "type": "str",
                                                     "choices": ["enable", "disable"]},
                                "auto-power-low": {"required": False, "type": "int"},
                                "band": {"required": False, "type": "str",
                                         "choices": ["802.11a", "802.11b", "802.11g",
                                                     "802.11n", "802.11n-5G", "802.11n,g-only",
                                                     "802.11g-only", "802.11n-only", "802.11n-5G-only",
                                                     "802.11ac", "802.11ac,n-only", "802.11ac-only"]},
                                "channel": {"required": False, "type": "list",
                                            "options": {
                                                "chan": {"required": True, "type": "str"}
                                            }},
                                "override-analysis": {"required": False, "type": "str",
                                                      "choices": ["enable", "disable"]},
                                "override-band": {"required": False, "type": "str",
                                                  "choices": ["enable", "disable"]},
                                "override-channel": {"required": False, "type": "str",
                                                     "choices": ["enable", "disable"]},
                                "override-txpower": {"required": False, "type": "str",
                                                     "choices": ["enable", "disable"]},
                                "override-vaps": {"required": False, "type": "str",
                                                  "choices": ["enable", "disable"]},
                                "power-level": {"required": False, "type": "int"},
                                "radio-id": {"required": False, "type": "int"},
                                "spectrum-analysis": {"required": False, "type": "str",
                                                      "choices": ["enable", "disable"]},
                                "vap-all": {"required": False, "type": "str",
                                            "choices": ["enable", "disable"]},
                                "vaps": {"required": False, "type": "list",
                                         "options": {
                                             "name": {"required": True, "type": "str"}
                                         }}
                            }},
                "split-tunneling-acl": {"required": False, "type": "list",
                                        "options": {
                                            "dest-ip": {"required": False, "type": "str"},
                                            "id": {"required": True, "type": "int"}
                                        }},
                "split-tunneling-acl-local-ap-subnet": {"required": False, "type": "str",
                                                        "choices": ["enable", "disable"]},
                "split-tunneling-acl-path": {"required": False, "type": "str",
                                             "choices": ["tunnel", "local"]},
                "tun-mtu-downlink": {"required": False, "type": "int"},
                "tun-mtu-uplink": {"required": False, "type": "int"},
                "wan-port-mode": {"required": False, "type": "str",
                                  "choices": ["wan-lan", "wan-only"]},
                "wtp-id": {"required": True, "type": "str"},
                "wtp-mode": {"required": False, "type": "str",
                             "choices": ["normal", "remote"]},
                "wtp-profile": {"required": False, "type": "str"}

            }
        }
    }

    module = AnsibleModule(argument_spec=fields,
                           supports_check_mode=False)
    try:
        from fortiosapi import FortiOSAPI
    except ImportError:
        module.fail_json(msg="fortiosapi module is required")

    fos = FortiOSAPI()

    is_error, has_changed, result = fortios_wireless_controller(module.params, fos)

    if not is_error:
        module.exit_json(changed=has_changed, meta=result)
    else:
        module.fail_json(msg="Error in repo", meta=result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2019, Dag Wieers (@dagwieers) <dag@wieers.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: mso_schema_site_anp_epg
short_description: Manage site-local Endpoint Groups (EPGs) in schema template
description:
- Manage site-local EPGs in schema template on Cisco ACI Multi-Site.
author:
- Dag Wieers (@dagwieers)
version_added: '2.8'
options:
  schema:
    description:
    - The name of the schema.
    type: str
    required: yes
  site:
    description:
    - The name of the site.
    type: str
    required: yes
  template:
    description:
    - The name of the template.
    type: str
    required: yes
  anp:
    description:
    - The name of the ANP.
    type: str
  epg:
    description:
    - The name of the EPG to manage.
    type: str
    aliases: [ name ]
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
seealso:
- module: mso_schema_site_anp
- module: mso_schema_site_anp_epg_subnet
- module: mso_schema_template_anp_epg
extends_documentation_fragment: mso
'''

EXAMPLES = r'''
- name: Add a new site EPG
  mso_schema_site_anp_epg:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema1
    site: Site1
    template: Template1
    anp: ANP1
    epg: EPG1
    state: present
  delegate_to: localhost

- name: Remove a site EPG
  mso_schema_site_anp_epg:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema1
    site: Site1
    template: Template1
    anp: ANP1
    epg: EPG1
    state: absent
  delegate_to: localhost

- name: Query a specific site EPGs
  mso_schema_site_anp_epg:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema1
    site: Site1
    template: Template1
    anp: ANP1
    epg: EPG1
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all site EPGs
  mso_schema_site_anp_epg:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema1
    site: Site1
    template: Template1
    anp: ANP1
    state: query
  delegate_to: localhost
  register: query_result
'''

RETURN = r'''
'''

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.network.aci.mso import MSOModule, mso_argument_spec, issubset


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        schema=dict(type='str', required=True),
        site=dict(type='str', required=True),
        template=dict(type='str', required=True),
        anp=dict(type='str', required=True),
        epg=dict(type='str', aliases=['name']),  # This parameter is not required for querying all objects
        state=dict(type='str', default='present', choices=['absent', 'present', 'query']),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ['state', 'absent', ['epg']],
            ['state', 'present', ['epg']],
        ],
    )

    schema = module.params['schema']
    site = module.params['site']
    template = module.params['template']
    anp = module.params['anp']
    epg = module.params['epg']
    state = module.params['state']

    mso = MSOModule(module)

    # Get schema_id
    schema_obj = mso.get_obj('schemas', displayName=schema)
    if not schema_obj:
        mso.fail_json(msg="Provided schema '{0}' does not exist".format(schema))

    schema_path = 'schemas/{id}'.format(**schema_obj)
    schema_id = schema_obj['id']

    # Get site
    site_id = mso.lookup_site(site)
    sites = [(s['siteId'], s['templateName']) for s in schema_obj['sites']]
    if (site_id, template) not in sites:
        mso.fail_json(msg="Provided site/template '{0}-{1}' does not exist. Existing sites/templates: {2}".format(site, template, ', '.join(sites)))

    # Schema-access uses indexes
    site_idx = sites.index((site_id, template))
    # Path-based access uses site_id-template
    site_template = '{0}-{1}'.format(site_id, template)

    # Get ANP
    anp_ref = mso.anp_ref(schema_id=schema_id, template=template, anp=anp)
    anps = [a['anpRef'] for a in schema_obj['sites'][site_idx]['anps']]
    if anp_ref not in anps:
        mso.fail_json(msg="Provided anp '{0}' does not exist. Existing anps: {1}".format(anp, ', '.join(anps)))
    anp_idx = anps.index(anp_ref)

    # Get EPG
    epg_ref = mso.epg_ref(schema_id=schema_id, template=template, anp=anp, epg=epg)
    epgs = [e['epgRef'] for e in schema_obj['sites'][site_idx]['anps'][anp_idx]['epgs']]
    if epg is not None and epg_ref in epgs:
        epg_idx = epgs.index(epg_ref)
        # FIXME: Changes based on index are DANGEROUS
        epg_path = '/sites/{0}/anps/{1}/epgs/{2}'.format(site_template, anp_idx, epg_idx)
        mso.existing = schema_obj['sites'][site_idx]['anps'][anp_idx]['epgs'][epg_idx]

    if state == 'query':
        if epg is None:
            mso.existing = schema_obj['sites'][site_idx]['anps'][anp_idx]['epgs']
        elif not mso.existing:
            mso.fail_json(msg="EPG '{epg}' not found".format(epg=epg))
        mso.exit_json()

    epgs_path = '/sites/{0}/anps/{1}/epgs'.format(site_template, anp_idx)
    ops = []

    mso.previous = mso.existing
    if state == 'absent':
        if mso.existing:
            mso.sent = mso.existing = {}
            ops.append(dict(op='remove', path=epg_path))

    elif state == 'present':

        payload = dict(
            epgRef=dict(
                schemaId=schema_id,
                templateName=template,
                anpName=anp,
                epgName=epg,
            ),
        )

        mso.sanitize(payload, collate=True)

        if not mso.existing:
            ops.append(dict(op='add', path=epgs_path + '/-', value=mso.sent))

        mso.existing = mso.proposed

    if not module.check_mode:
        mso.request(schema_path, method='PATCH', data=ops)

    mso.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
from __future__ import (absolute_import, division, print_function)
# Copyright 2019 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# the lib use python logging can get it if the following is set in your
# Ansible config.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fortios_router_multicast
short_description: Configure router multicast in Fortinet's FortiOS and FortiGate.
description:
    - This module is able to configure a FortiGate or FortiOS by allowing the
      user to set and modify router feature and multicast category.
      Examples include all parameters and values need to be adjusted to datasources before usage.
      Tested with FOS v6.0.2
version_added: "2.8"
author:
    - Miguel Angel Munoz (@mamunozgonzalez)
    - Nicolas Thomas (@thomnico)
notes:
    - Requires fortiosapi library developed by Fortinet
    - Run as a local_action in your playbook
requirements:
    - fortiosapi>=0.9.8
options:
    host:
       description:
            - FortiOS or FortiGate ip address.
       required: true
    username:
        description:
            - FortiOS or FortiGate username.
        required: true
    password:
        description:
            - FortiOS or FortiGate password.
        default: ""
    vdom:
        description:
            - Virtual domain, among those defined previously. A vdom is a
              virtual instance of the FortiGate that can be configured and
              used as a different unit.
        default: root
    https:
        description:
            - Indicates if the requests towards FortiGate must use HTTPS
              protocol
        type: bool
        default: true
    router_multicast:
        description:
            - Configure router multicast.
        default: null
        suboptions:
            interface:
                description:
                    - PIM interfaces.
                suboptions:
                    bfd:
                        description:
                            - Enable/disable Protocol Independent Multicast (PIM) Bidirectional Forwarding Detection (BFD).
                        choices:
                            - enable
                            - disable
                    cisco-exclude-genid:
                        description:
                            - Exclude GenID from hello packets (compatibility with old Cisco IOS).
                        choices:
                            - enable
                            - disable
                    dr-priority:
                        description:
                            - DR election priority.
                    hello-holdtime:
                        description:
                            - Time before old neighbor information expires (0 - 65535 sec, default = 105).
                    hello-interval:
                        description:
                            - Interval between sending PIM hello messages (0 - 65535 sec, default = 30).
                    igmp:
                        description:
                            - IGMP configuration options.
                        suboptions:
                            access-group:
                                description:
                                    - Groups IGMP hosts are allowed to join. Source router.access-list.name.
                            immediate-leave-group:
                                description:
                                    - Groups to drop membership for immediately after receiving IGMPv2 leave. Source router.access-list.name.
                            last-member-query-count:
                                description:
                                    - Number of group specific queries before removing group (2 - 7, default = 2).
                            last-member-query-interval:
                                description:
                                    - Timeout between IGMPv2 leave and removing group (1 - 65535 msec, default = 1000).
                            query-interval:
                                description:
                                    - Interval between queries to IGMP hosts (1 - 65535 sec, default = 125).
                            query-max-response-time:
                                description:
                                    - Maximum time to wait for a IGMP query response (1 - 25 sec, default = 10).
                            query-timeout:
                                description:
                                    - Timeout between queries before becoming querier for network (60 - 900, default = 255).
                            router-alert-check:
                                description:
                                    - Enable/disable require IGMP packets contain router alert option.
                                choices:
                                    - enable
                                    - disable
                            version:
                                description:
                                    - Maximum version of IGMP to support.
                                choices:
                                    - 3
                                    - 2
                                    - 1
                    join-group:
                        description:
                            - Join multicast groups.
                        suboptions:
                            address:
                                description:
                                    - Multicast group IP address.
                                required: true
                    multicast-flow:
                        description:
                            - Acceptable source for multicast group. Source router.multicast-flow.name.
                    name:
                        description:
                            - Interface name. Source system.interface.name.
                        required: true
                    neighbour-filter:
                        description:
                            - Routers acknowledged as neighbor routers. Source router.access-list.name.
                    passive:
                        description:
                            - Enable/disable listening to IGMP but not participating in PIM.
                        choices:
                            - enable
                            - disable
                    pim-mode:
                        description:
                            - PIM operation mode.
                        choices:
                            - sparse-mode
                            - dense-mode
                    propagation-delay:
                        description:
                            - Delay flooding packets on this interface (100 - 5000 msec, default = 500).
                    rp-candidate:
                        description:
                            - Enable/disable compete to become RP in elections.
                        choices:
                            - enable
                            - disable
                    rp-candidate-group:
                        description:
                            - Multicast groups managed by this RP. Source router.access-list.name.
                    rp-candidate-interval:
                        description:
                            - RP candidate advertisement interval (1 - 16383 sec, default = 60).
                    rp-candidate-priority:
                        description:
                            - Router's priority as RP.
                    state-refresh-interval:
                        description:
                            - Interval between sending state-refresh packets (1 - 100 sec, default = 60).
                    static-group:
                        description:
                            - Statically set multicast groups to forward out. Source router.multicast-flow.name.
                    ttl-threshold:
                        description:
                            - Minimum TTL of multicast packets that will be forwarded (applied only to new multicast routes) (1 - 255, default = 1).
            multicast-routing:
                description:
                    - Enable/disable IP multicast routing.
                choices:
                    - enable
                    - disable
            pim-sm-global:
                description:
                    - PIM sparse-mode global settings.
                suboptions:
                    accept-register-list:
                        description:
                            - Sources allowed to register packets with this Rendezvous Point (RP). Source router.access-list.name.
                    accept-source-list:
                        description:
                            - Sources allowed to send multicast traffic. Source router.access-list.name.
                    bsr-allow-quick-refresh:
                        description:
                            - Enable/disable accept BSR quick refresh packets from neighbors.
                        choices:
                            - enable
                            - disable
                    bsr-candidate:
                        description:
                            - Enable/disable allowing this router to become a bootstrap router (BSR).
                        choices:
                            - enable
                            - disable
                    bsr-hash:
                        description:
                            - BSR hash length (0 - 32, default = 10).
                    bsr-interface:
                        description:
                            - Interface to advertise as candidate BSR. Source system.interface.name.
                    bsr-priority:
                        description:
                            - BSR priority (0 - 255, default = 0).
                    cisco-crp-prefix:
                        description:
                            - Enable/disable making candidate RP compatible with old Cisco IOS.
                        choices:
                            - enable
                            - disable
                    cisco-ignore-rp-set-priority:
                        description:
                            - Use only hash for RP selection (compatibility with old Cisco IOS).
                        choices:
                            - enable
                            - disable
                    cisco-register-checksum:
                        description:
                            - Checksum entire register packet(for old Cisco IOS compatibility).
                        choices:
                            - enable
                            - disable
                    cisco-register-checksum-group:
                        description:
                            - Cisco register checksum only these groups. Source router.access-list.name.
                    join-prune-holdtime:
                        description:
                            - Join/prune holdtime (1 - 65535, default = 210).
                    message-interval:
                        description:
                            - Period of time between sending periodic PIM join/prune messages in seconds (1 - 65535, default = 60).
                    null-register-retries:
                        description:
                            - Maximum retries of null register (1 - 20, default = 1).
                    register-rate-limit:
                        description:
                            - Limit of packets/sec per source registered through this RP (0 - 65535, default = 0 which means unlimited).
                    register-rp-reachability:
                        description:
                            - Enable/disable check RP is reachable before registering packets.
                        choices:
                            - enable
                            - disable
                    register-source:
                        description:
                            - Override source address in register packets.
                        choices:
                            - disable
                            - interface
                            - ip-address
                    register-source-interface:
                        description:
                            - Override with primary interface address. Source system.interface.name.
                    register-source-ip:
                        description:
                            - Override with local IP address.
                    register-supression:
                        description:
                            - Period of time to honor register-stop message (1 - 65535 sec, default = 60).
                    rp-address:
                        description:
                            - Statically configure RP addresses.
                        suboptions:
                            group:
                                description:
                                    - Groups to use this RP. Source router.access-list.name.
                            id:
                                description:
                                    - ID.
                                required: true
                            ip-address:
                                description:
                                    - RP router address.
                    rp-register-keepalive:
                        description:
                            - Timeout for RP receiving data on (S,G) tree (1 - 65535 sec, default = 185).
                    spt-threshold:
                        description:
                            - Enable/disable switching to source specific trees.
                        choices:
                            - enable
                            - disable
                    spt-threshold-group:
                        description:
                            - Groups allowed to switch to source tree. Source router.access-list.name.
                    ssm:
                        description:
                            - Enable/disable source specific multicast.
                        choices:
                            - enable
                            - disable
                    ssm-range:
                        description:
                            - Groups allowed to source specific multicast. Source router.access-list.name.
            route-limit:
                description:
                    - Maximum number of multicast routes.
            route-threshold:
                description:
                    - Generate warnings when the number of multicast routes exceeds this number, must not be greater than route-limit.
'''

EXAMPLES = '''
- hosts: localhost
  vars:
   host: "192.168.122.40"
   username: "admin"
   password: ""
   vdom: "root"
  tasks:
  - name: Configure router multicast.
    fortios_router_multicast:
      host:  "{{ host }}"
      username: "{{ username }}"
      password: "{{ password }}"
      vdom:  "{{ vdom }}"
      https: "False"
      router_multicast:
        interface:
         -
            bfd: "enable"
            cisco-exclude-genid: "enable"
            dr-priority: "6"
            hello-holdtime: "7"
            hello-interval: "8"
            igmp:
                access-group: "<your_own_value> (source router.access-list.name)"
                immediate-leave-group: "<your_own_value> (source router.access-list.name)"
                last-member-query-count: "12"
                last-member-query-interval: "13"
                query-interval: "14"
                query-max-response-time: "15"
                query-timeout: "16"
                router-alert-check: "enable"
                version: "3"
            join-group:
             -
                address: "<your_own_value>"
            multicast-flow: "<your_own_value> (source router.multicast-flow.name)"
            name: "default_name_22 (source system.interface.name)"
            neighbour-filter: "<your_own_value> (source router.access-list.name)"
            passive: "enable"
            pim-mode: "sparse-mode"
            propagation-delay: "26"
            rp-candidate: "enable"
            rp-candidate-group: "<your_own_value> (source router.access-list.name)"
            rp-candidate-interval: "29"
            rp-candidate-priority: "30"
            state-refresh-interval: "31"
            static-group: "<your_own_value> (source router.multicast-flow.name)"
            ttl-threshold: "33"
        multicast-routing: "enable"
        pim-sm-global:
            accept-register-list: "<your_own_value> (source router.access-list.name)"
            accept-source-list: "<your_own_value> (source router.access-list.name)"
            bsr-allow-quick-refresh: "enable"
            bsr-candidate: "enable"
            bsr-hash: "40"
            bsr-interface: "<your_own_value> (source system.interface.name)"
            bsr-priority: "42"
            cisco-crp-prefix: "enable"
            cisco-ignore-rp-set-priority: "enable"
            cisco-register-checksum: "enable"
            cisco-register-checksum-group: "<your_own_value> (source router.access-list.name)"
            join-prune-holdtime: "47"
            message-interval: "48"
            null-register-retries: "49"
            register-rate-limit: "50"
            register-rp-reachability: "enable"
            register-source: "disable"
            register-source-interface: "<your_own_value> (source system.interface.name)"
            register-source-ip: "<your_own_value>"
            register-supression: "55"
            rp-address:
             -
                group: "<your_own_value> (source router.access-list.name)"
                id:  "58"
                ip-address: "<your_own_value>"
            rp-register-keepalive: "60"
            spt-threshold: "enable"
            spt-threshold-group: "<your_own_value> (source router.access-list.name)"
            ssm: "enable"
            ssm-range: "<your_own_value> (source router.access-list.name)"
        route-limit: "65"
        route-threshold: "66"
'''

RETURN = '''
build:
  description: Build number of the fortigate image
  returned: always
  type: str
  sample: '1547'
http_method:
  description: Last method used to provision the content into FortiGate
  returned: always
  type: str
  sample: 'PUT'
http_status:
  description: Last result given by FortiGate on last operation applied
  returned: always
  type: str
  sample: "200"
mkey:
  description: Master key (id) used in the last call to FortiGate
  returned: success
  type: str
  sample: "id"
name:
  description: Name of the table used to fulfill the request
  returned: always
  type: str
  sample: "urlfilter"
path:
  description: Path of the table used to fulfill the request
  returned: always
  type: str
  sample: "webfilter"
revision:
  description: Internal revision number
  returned: always
  type: str
  sample: "17.0.2.10658"
serial:
  description: Serial number of the unit
  returned: always
  type: str
  sample: "FGVMEVYYQT3AB5352"
status:
  description: Indication of the operation's result
  returned: always
  type: str
  sample: "success"
vdom:
  description: Virtual domain used
  returned: always
  type: str
  sample: "root"
version:
  description: Version of the FortiGate
  returned: always
  type: str
  sample: "v5.6.3"

'''

from ansible.module_utils.basic import AnsibleModule

fos = None


def login(data):
    host = data['host']
    username = data['username']
    password = data['password']

    fos.debug('on')
    if 'https' in data and not data['https']:
        fos.https('off')
    else:
        fos.https('on')

    fos.login(host, username, password)


def filter_router_multicast_data(json):
    option_list = ['interface', 'multicast-routing', 'pim-sm-global',
                   'route-limit', 'route-threshold']
    dictionary = {}

    for attribute in option_list:
        if attribute in json and json[attribute] is not None:
            dictionary[attribute] = json[attribute]

    return dictionary


def flatten_multilists_attributes(data):
    multilist_attrs = []

    for attr in multilist_attrs:
        try:
            path = "data['" + "']['".join(elem for elem in attr) + "']"
            current_val = eval(path)
            flattened_val = ' '.join(elem for elem in current_val)
            exec(path + '= flattened_val')
        except BaseException:
            pass

    return data


def router_multicast(data, fos):
    vdom = data['vdom']
    router_multicast_data = data['router_multicast']
    flattened_data = flatten_multilists_attributes(router_multicast_data)
    filtered_data = filter_router_multicast_data(flattened_data)
    return fos.set('router',
                   'multicast',
                   data=filtered_data,
                   vdom=vdom)


def fortios_router(data, fos):
    login(data)

    if data['router_multicast']:
        resp = router_multicast(data, fos)

    fos.logout()
    return not resp['status'] == "success", resp['status'] == "success", resp


def main():
    fields = {
        "host": {"required": True, "type": "str"},
        "username": {"required": True, "type": "str"},
        "password": {"required": False, "type": "str", "no_log": True},
        "vdom": {"required": False, "type": "str", "default": "root"},
        "https": {"required": False, "type": "bool", "default": True},
        "router_multicast": {
            "required": False, "type": "dict",
            "options": {
                "interface": {"required": False, "type": "list",
                              "options": {
                                  "bfd": {"required": False, "type": "str",
                                          "choices": ["enable", "disable"]},
                                  "cisco-exclude-genid": {"required": False, "type": "str",
                                                          "choices": ["enable", "disable"]},
                                  "dr-priority": {"required": False, "type": "int"},
                                  "hello-holdtime": {"required": False, "type": "int"},
                                  "hello-interval": {"required": False, "type": "int"},
                                  "igmp": {"required": False, "type": "dict",
                                           "options": {
                                               "access-group": {"required": False, "type": "str"},
                                               "immediate-leave-group": {"required": False, "type": "str"},
                                               "last-member-query-count": {"required": False, "type": "int"},
                                               "last-member-query-interval": {"required": False, "type": "int"},
                                               "query-interval": {"required": False, "type": "int"},
                                               "query-max-response-time": {"required": False, "type": "int"},
                                               "query-timeout": {"required": False, "type": "int"},
                                               "router-alert-check": {"required": False, "type": "str",
                                                                      "choices": ["enable", "disable"]},
                                               "version": {"required": False, "type": "str",
                                                           "choices": ["3", "2", "1"]}
                                           }},
                                  "join-group": {"required": False, "type": "list",
                                                 "options": {
                                                     "address": {"required": True, "type": "str"}
                                                 }},
                                  "multicast-flow": {"required": False, "type": "str"},
                                  "name": {"required": True, "type": "str"},
                                  "neighbour-filter": {"required": False, "type": "str"},
                                  "passive": {"required": False, "type": "str",
                                              "choices": ["enable", "disable"]},
                                  "pim-mode": {"required": False, "type": "str",
                                               "choices": ["sparse-mode", "dense-mode"]},
                                  "propagation-delay": {"required": False, "type": "int"},
                                  "rp-candidate": {"required": False, "type": "str",
                                                   "choices": ["enable", "disable"]},
                                  "rp-candidate-group": {"required": False, "type": "str"},
                                  "rp-candidate-interval": {"required": False, "type": "int"},
                                  "rp-candidate-priority": {"required": False, "type": "int"},
                                  "state-refresh-interval": {"required": False, "type": "int"},
                                  "static-group": {"required": False, "type": "str"},
                                  "ttl-threshold": {"required": False, "type": "int"}
                              }},
                "multicast-routing": {"required": False, "type": "str",
                                      "choices": ["enable", "disable"]},
                "pim-sm-global": {"required": False, "type": "dict",
                                  "options": {
                                      "accept-register-list": {"required": False, "type": "str"},
                                      "accept-source-list": {"required": False, "type": "str"},
                                      "bsr-allow-quick-refresh": {"required": False, "type": "str",
                                                                  "choices": ["enable", "disable"]},
                                      "bsr-candidate": {"required": False, "type": "str",
                                                        "choices": ["enable", "disable"]},
                                      "bsr-hash": {"required": False, "type": "int"},
                                      "bsr-interface": {"required": False, "type": "str"},
                                      "bsr-priority": {"required": False, "type": "int"},
                                      "cisco-crp-prefix": {"required": False, "type": "str",
                                                           "choices": ["enable", "disable"]},
                                      "cisco-ignore-rp-set-priority": {"required": False, "type": "str",
                                                                       "choices": ["enable", "disable"]},
                                      "cisco-register-checksum": {"required": False, "type": "str",
                                                                  "choices": ["enable", "disable"]},
                                      "cisco-register-checksum-group": {"required": False, "type": "str"},
                                      "join-prune-holdtime": {"required": False, "type": "int"},
                                      "message-interval": {"required": False, "type": "int"},
                                      "null-register-retries": {"required": False, "type": "int"},
                                      "register-rate-limit": {"required": False, "type": "int"},
                                      "register-rp-reachability": {"required": False, "type": "str",
                                                                   "choices": ["enable", "disable"]},
                                      "register-source": {"required": False, "type": "str",
                                                          "choices": ["disable", "interface", "ip-address"]},
                                      "register-source-interface": {"required": False, "type": "str"},
                                      "register-source-ip": {"required": False, "type": "str"},
                                      "register-supression": {"required": False, "type": "int"},
                                      "rp-address": {"required": False, "type": "list",
                                                     "options": {
                                                         "group": {"required": False, "type": "str"},
                                                         "id": {"required": True, "type": "int"},
                                                         "ip-address": {"required": False, "type": "str"}
                                                     }},
                                      "rp-register-keepalive": {"required": False, "type": "int"},
                                      "spt-threshold": {"required": False, "type": "str",
                                                        "choices": ["enable", "disable"]},
                                      "spt-threshold-group": {"required": False, "type": "str"},
                                      "ssm": {"required": False, "type": "str",
                                              "choices": ["enable", "disable"]},
                                      "ssm-range": {"required": False, "type": "str"}
                                  }},
                "route-limit": {"required": False, "type": "int"},
                "route-threshold": {"required": False, "type": "int"}

            }
        }
    }

    module = AnsibleModule(argument_spec=fields,
                           supports_check_mode=False)
    try:
        from fortiosapi import FortiOSAPI
    except ImportError:
        module.fail_json(msg="fortiosapi module is required")

    global fos
    fos = FortiOSAPI()

    is_error, has_changed, result = fortios_router(module.params, fos)

    if not is_error:
        module.exit_json(changed=has_changed, meta=result)
    else:
        module.fail_json(msg="Error in repo", meta=result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# (c) 2018, Yanis Guenane <yanis+ansible@guenane.org>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: vultr_firewall_group_facts
short_description: Gather facts about the Vultr firewall groups available.
description:
  - Gather facts about firewall groups available in Vultr.
version_added: "2.7"
author: "Yanis Guenane (@Spredzy)"
extends_documentation_fragment: vultr
'''

EXAMPLES = r'''
- name: Gather Vultr firewall groups facts
  local_action:
    module: vultr_firewall_group_facts

- name: Print the gathered facts
  debug:
    var: ansible_facts.vultr_firewall_group_facts
'''

RETURN = r'''
---
vultr_api:
  description: Response from Vultr API with a few additions/modification
  returned: success
  type: complex
  contains:
    api_account:
      description: Account used in the ini file to select the key
      returned: success
      type: str
      sample: default
    api_timeout:
      description: Timeout used for the API requests
      returned: success
      type: int
      sample: 60
    api_retries:
      description: Amount of max retries for the API requests
      returned: success
      type: int
      sample: 5
    api_endpoint:
      description: Endpoint used for the API requests
      returned: success
      type: str
      sample: "https://api.vultr.com"
vultr_firewall_group_facts:
  description: Response from Vultr API
  returned: success
  type: complex
  sample:
    "vultr_firewall_group_facts": [
      {
        "date_created": "2018-07-12 10:27:14",
        "date_modified": "2018-07-12 10:27:14",
        "description": "test",
        "id": "5e128ff0",
        "instance_count": 0,
        "max_rule_count": 50,
        "rule_count": 0
      }
    ]
'''

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.vultr import (
    Vultr,
    vultr_argument_spec,
)


class AnsibleVultrFirewallGroupFacts(Vultr):

    def __init__(self, module):
        super(AnsibleVultrFirewallGroupFacts, self).__init__(module, "vultr_firewall_group_facts")

        self.returns = {
            "FIREWALLGROUPID": dict(key='id'),
            "date_created": dict(),
            "date_modified": dict(),
            "description": dict(),
            "instance_count": dict(convert_to='int'),
            "max_rule_count": dict(convert_to='int'),
            "rule_count": dict(convert_to='int')
        }

    def get_firewall_group(self):
        return self.api_query(path="/v1/firewall/group_list")


def parse_fw_group_list(fwgroups_list):
    if not fwgroups_list:
        return []

    return [group for id, group in fwgroups_list.items()]


def main():
    argument_spec = vultr_argument_spec()

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    fw_group_facts = AnsibleVultrFirewallGroupFacts(module)
    result = fw_group_facts.get_result(parse_fw_group_list(fw_group_facts.get_firewall_group()))
    ansible_facts = {
        'vultr_firewall_group_facts': result['vultr_firewall_group_facts']
    }
    module.exit_json(ansible_facts=ansible_facts, **result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
from __future__ import (absolute_import, division, print_function)
# Copyright 2019 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# the lib use python logging can get it if the following is set in your
# Ansible config.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fortios_router_ospf
short_description: Configure OSPF in Fortinet's FortiOS and FortiGate.
description:
    - This module is able to configure a FortiGate or FortiOS by allowing the
      user to set and modify router feature and ospf category.
      Examples include all parameters and values need to be adjusted to datasources before usage.
      Tested with FOS v6.0.2
version_added: "2.8"
author:
    - Miguel Angel Munoz (@mamunozgonzalez)
    - Nicolas Thomas (@thomnico)
notes:
    - Requires fortiosapi library developed by Fortinet
    - Run as a local_action in your playbook
requirements:
    - fortiosapi>=0.9.8
options:
    host:
       description:
            - FortiOS or FortiGate ip address.
       required: true
    username:
        description:
            - FortiOS or FortiGate username.
        required: true
    password:
        description:
            - FortiOS or FortiGate password.
        default: ""
    vdom:
        description:
            - Virtual domain, among those defined previously. A vdom is a
              virtual instance of the FortiGate that can be configured and
              used as a different unit.
        default: root
    https:
        description:
            - Indicates if the requests towards FortiGate must use HTTPS
              protocol
        type: bool
        default: true
    router_ospf:
        description:
            - Configure OSPF.
        default: null
        suboptions:
            abr-type:
                description:
                    - Area border router type.
                choices:
                    - cisco
                    - ibm
                    - shortcut
                    - standard
            area:
                description:
                    - OSPF area configuration.
                suboptions:
                    authentication:
                        description:
                            - Authentication type.
                        choices:
                            - none
                            - text
                            - md5
                    default-cost:
                        description:
                            - Summary default cost of stub or NSSA area.
                    filter-list:
                        description:
                            - OSPF area filter-list configuration.
                        suboptions:
                            direction:
                                description:
                                    - Direction.
                                choices:
                                    - in
                                    - out
                            id:
                                description:
                                    - Filter list entry ID.
                                required: true
                            list:
                                description:
                                    - Access-list or prefix-list name. Source router.access-list.name router.prefix-list.name.
                    id:
                        description:
                            - Area entry IP address.
                        required: true
                    nssa-default-information-originate:
                        description:
                            - Redistribute, advertise, or do not originate Type-7 default route into NSSA area.
                        choices:
                            - enable
                            - always
                            - disable
                    nssa-default-information-originate-metric:
                        description:
                            - OSPF default metric.
                    nssa-default-information-originate-metric-type:
                        description:
                            - OSPF metric type for default routes.
                        choices:
                            - 1
                            - 2
                    nssa-redistribution:
                        description:
                            - Enable/disable redistribute into NSSA area.
                        choices:
                            - enable
                            - disable
                    nssa-translator-role:
                        description:
                            - NSSA translator role type.
                        choices:
                            - candidate
                            - never
                            - always
                    range:
                        description:
                            - OSPF area range configuration.
                        suboptions:
                            advertise:
                                description:
                                    - Enable/disable advertise status.
                                choices:
                                    - disable
                                    - enable
                            id:
                                description:
                                    - Range entry ID.
                                required: true
                            prefix:
                                description:
                                    - Prefix.
                            substitute:
                                description:
                                    - Substitute prefix.
                            substitute-status:
                                description:
                                    - Enable/disable substitute status.
                                choices:
                                    - enable
                                    - disable
                    shortcut:
                        description:
                            - Enable/disable shortcut option.
                        choices:
                            - disable
                            - enable
                            - default
                    stub-type:
                        description:
                            - Stub summary setting.
                        choices:
                            - no-summary
                            - summary
                    type:
                        description:
                            - Area type setting.
                        choices:
                            - regular
                            - nssa
                            - stub
                    virtual-link:
                        description:
                            - OSPF virtual link configuration.
                        suboptions:
                            authentication:
                                description:
                                    - Authentication type.
                                choices:
                                    - none
                                    - text
                                    - md5
                            authentication-key:
                                description:
                                    - Authentication key.
                            dead-interval:
                                description:
                                    - Dead interval.
                            hello-interval:
                                description:
                                    - Hello interval.
                            md5-key:
                                description:
                                    - MD5 key.
                            name:
                                description:
                                    - Virtual link entry name.
                                required: true
                            peer:
                                description:
                                    - Peer IP.
                            retransmit-interval:
                                description:
                                    - Retransmit interval.
                            transmit-delay:
                                description:
                                    - Transmit delay.
            auto-cost-ref-bandwidth:
                description:
                    - Reference bandwidth in terms of megabits per second.
            bfd:
                description:
                    - Bidirectional Forwarding Detection (BFD).
                choices:
                    - enable
                    - disable
            database-overflow:
                description:
                    - Enable/disable database overflow.
                choices:
                    - enable
                    - disable
            database-overflow-max-lsas:
                description:
                    - Database overflow maximum LSAs.
            database-overflow-time-to-recover:
                description:
                    - Database overflow time to recover (sec).
            default-information-metric:
                description:
                    - Default information metric.
            default-information-metric-type:
                description:
                    - Default information metric type.
                choices:
                    - 1
                    - 2
            default-information-originate:
                description:
                    - Enable/disable generation of default route.
                choices:
                    - enable
                    - always
                    - disable
            default-information-route-map:
                description:
                    - Default information route map. Source router.route-map.name.
            default-metric:
                description:
                    - Default metric of redistribute routes.
            distance:
                description:
                    - Distance of the route.
            distance-external:
                description:
                    - Administrative external distance.
            distance-inter-area:
                description:
                    - Administrative inter-area distance.
            distance-intra-area:
                description:
                    - Administrative intra-area distance.
            distribute-list:
                description:
                    - Distribute list configuration.
                suboptions:
                    access-list:
                        description:
                            - Access list name. Source router.access-list.name.
                    id:
                        description:
                            - Distribute list entry ID.
                        required: true
                    protocol:
                        description:
                            - Protocol type.
                        choices:
                            - connected
                            - static
                            - rip
            distribute-list-in:
                description:
                    - Filter incoming routes. Source router.access-list.name router.prefix-list.name.
            distribute-route-map-in:
                description:
                    - Filter incoming external routes by route-map. Source router.route-map.name.
            log-neighbour-changes:
                description:
                    - Enable logging of OSPF neighbour's changes
                choices:
                    - enable
                    - disable
            neighbor:
                description:
                    - OSPF neighbor configuration are used when OSPF runs on non-broadcast media
                suboptions:
                    cost:
                        description:
                            - Cost of the interface, value range from 0 to 65535, 0 means auto-cost.
                    id:
                        description:
                            - Neighbor entry ID.
                        required: true
                    ip:
                        description:
                            - Interface IP address of the neighbor.
                    poll-interval:
                        description:
                            - Poll interval time in seconds.
                    priority:
                        description:
                            - Priority.
            network:
                description:
                    - OSPF network configuration.
                suboptions:
                    area:
                        description:
                            - Attach the network to area.
                    id:
                        description:
                            - Network entry ID.
                        required: true
                    prefix:
                        description:
                            - Prefix.
            ospf-interface:
                description:
                    - OSPF interface configuration.
                suboptions:
                    authentication:
                        description:
                            - Authentication type.
                        choices:
                            - none
                            - text
                            - md5
                    authentication-key:
                        description:
                            - Authentication key.
                    bfd:
                        description:
                            - Bidirectional Forwarding Detection (BFD).
                        choices:
                            - global
                            - enable
                            - disable
                    cost:
                        description:
                            - Cost of the interface, value range from 0 to 65535, 0 means auto-cost.
                    database-filter-out:
                        description:
                            - Enable/disable control of flooding out LSAs.
                        choices:
                            - enable
                            - disable
                    dead-interval:
                        description:
                            - Dead interval.
                    hello-interval:
                        description:
                            - Hello interval.
                    hello-multiplier:
                        description:
                            - Number of hello packets within dead interval.
                    interface:
                        description:
                            - Configuration interface name. Source system.interface.name.
                    ip:
                        description:
                            - IP address.
                    md5-key:
                        description:
                            - MD5 key.
                    mtu:
                        description:
                            - MTU for database description packets.
                    mtu-ignore:
                        description:
                            - Enable/disable ignore MTU.
                        choices:
                            - enable
                            - disable
                    name:
                        description:
                            - Interface entry name.
                        required: true
                    network-type:
                        description:
                            - Network type.
                        choices:
                            - broadcast
                            - non-broadcast
                            - point-to-point
                            - point-to-multipoint
                            - point-to-multipoint-non-broadcast
                    prefix-length:
                        description:
                            - Prefix length.
                    priority:
                        description:
                            - Priority.
                    resync-timeout:
                        description:
                            - Graceful restart neighbor resynchronization timeout.
                    retransmit-interval:
                        description:
                            - Retransmit interval.
                    status:
                        description:
                            - Enable/disable status.
                        choices:
                            - disable
                            - enable
                    transmit-delay:
                        description:
                            - Transmit delay.
            passive-interface:
                description:
                    - Passive interface configuration.
                suboptions:
                    name:
                        description:
                            - Passive interface name. Source system.interface.name.
                        required: true
            redistribute:
                description:
                    - Redistribute configuration.
                suboptions:
                    metric:
                        description:
                            - Redistribute metric setting.
                    metric-type:
                        description:
                            - Metric type.
                        choices:
                            - 1
                            - 2
                    name:
                        description:
                            - Redistribute name.
                        required: true
                    routemap:
                        description:
                            - Route map name. Source router.route-map.name.
                    status:
                        description:
                            - status
                        choices:
                            - enable
                            - disable
                    tag:
                        description:
                            - Tag value.
            restart-mode:
                description:
                    - OSPF restart mode (graceful or LLS).
                choices:
                    - none
                    - lls
                    - graceful-restart
            restart-period:
                description:
                    - Graceful restart period.
            rfc1583-compatible:
                description:
                    - Enable/disable RFC1583 compatibility.
                choices:
                    - enable
                    - disable
            router-id:
                description:
                    - Router ID.
            spf-timers:
                description:
                    - SPF calculation frequency.
            summary-address:
                description:
                    - IP address summary configuration.
                suboptions:
                    advertise:
                        description:
                            - Enable/disable advertise status.
                        choices:
                            - disable
                            - enable
                    id:
                        description:
                            - Summary address entry ID.
                        required: true
                    prefix:
                        description:
                            - Prefix.
                    tag:
                        description:
                            - Tag value.
'''

EXAMPLES = '''
- hosts: localhost
  vars:
   host: "192.168.122.40"
   username: "admin"
   password: ""
   vdom: "root"
  tasks:
  - name: Configure OSPF.
    fortios_router_ospf:
      host:  "{{ host }}"
      username: "{{ username }}"
      password: "{{ password }}"
      vdom:  "{{ vdom }}"
      https: "False"
      router_ospf:
        abr-type: "cisco"
        area:
         -
            authentication: "none"
            default-cost: "6"
            filter-list:
             -
                direction: "in"
                id:  "9"
                list: "<your_own_value> (source router.access-list.name router.prefix-list.name)"
            id:  "11"
            nssa-default-information-originate: "enable"
            nssa-default-information-originate-metric: "13"
            nssa-default-information-originate-metric-type: "1"
            nssa-redistribution: "enable"
            nssa-translator-role: "candidate"
            range:
             -
                advertise: "disable"
                id:  "19"
                prefix: "<your_own_value>"
                substitute: "<your_own_value>"
                substitute-status: "enable"
            shortcut: "disable"
            stub-type: "no-summary"
            type: "regular"
            virtual-link:
             -
                authentication: "none"
                authentication-key: "<your_own_value>"
                dead-interval: "29"
                hello-interval: "30"
                md5-key: "<your_own_value>"
                name: "default_name_32"
                peer: "<your_own_value>"
                retransmit-interval: "34"
                transmit-delay: "35"
        auto-cost-ref-bandwidth: "36"
        bfd: "enable"
        database-overflow: "enable"
        database-overflow-max-lsas: "39"
        database-overflow-time-to-recover: "40"
        default-information-metric: "41"
        default-information-metric-type: "1"
        default-information-originate: "enable"
        default-information-route-map: "<your_own_value> (source router.route-map.name)"
        default-metric: "45"
        distance: "46"
        distance-external: "47"
        distance-inter-area: "48"
        distance-intra-area: "49"
        distribute-list:
         -
            access-list: "<your_own_value> (source router.access-list.name)"
            id:  "52"
            protocol: "connected"
        distribute-list-in: "<your_own_value> (source router.access-list.name router.prefix-list.name)"
        distribute-route-map-in: "<your_own_value> (source router.route-map.name)"
        log-neighbour-changes: "enable"
        neighbor:
         -
            cost: "58"
            id:  "59"
            ip: "<your_own_value>"
            poll-interval: "61"
            priority: "62"
        network:
         -
            area: "<your_own_value>"
            id:  "65"
            prefix: "<your_own_value>"
        ospf-interface:
         -
            authentication: "none"
            authentication-key: "<your_own_value>"
            bfd: "global"
            cost: "71"
            database-filter-out: "enable"
            dead-interval: "73"
            hello-interval: "74"
            hello-multiplier: "75"
            interface: "<your_own_value> (source system.interface.name)"
            ip: "<your_own_value>"
            md5-key: "<your_own_value>"
            mtu: "79"
            mtu-ignore: "enable"
            name: "default_name_81"
            network-type: "broadcast"
            prefix-length: "83"
            priority: "84"
            resync-timeout: "85"
            retransmit-interval: "86"
            status: "disable"
            transmit-delay: "88"
        passive-interface:
         -
            name: "default_name_90 (source system.interface.name)"
        redistribute:
         -
            metric: "92"
            metric-type: "1"
            name: "default_name_94"
            routemap: "<your_own_value> (source router.route-map.name)"
            status: "enable"
            tag: "97"
        restart-mode: "none"
        restart-period: "99"
        rfc1583-compatible: "enable"
        router-id: "<your_own_value>"
        spf-timers: "<your_own_value>"
        summary-address:
         -
            advertise: "disable"
            id:  "105"
            prefix: "<your_own_value>"
            tag: "107"
'''

RETURN = '''
build:
  description: Build number of the fortigate image
  returned: always
  type: str
  sample: '1547'
http_method:
  description: Last method used to provision the content into FortiGate
  returned: always
  type: str
  sample: 'PUT'
http_status:
  description: Last result given by FortiGate on last operation applied
  returned: always
  type: str
  sample: "200"
mkey:
  description: Master key (id) used in the last call to FortiGate
  returned: success
  type: str
  sample: "id"
name:
  description: Name of the table used to fulfill the request
  returned: always
  type: str
  sample: "urlfilter"
path:
  description: Path of the table used to fulfill the request
  returned: always
  type: str
  sample: "webfilter"
revision:
  description: Internal revision number
  returned: always
  type: str
  sample: "17.0.2.10658"
serial:
  description: Serial number of the unit
  returned: always
  type: str
  sample: "FGVMEVYYQT3AB5352"
status:
  description: Indication of the operation's result
  returned: always
  type: str
  sample: "success"
vdom:
  description: Virtual domain used
  returned: always
  type: str
  sample: "root"
version:
  description: Version of the FortiGate
  returned: always
  type: str
  sample: "v5.6.3"

'''

from ansible.module_utils.basic import AnsibleModule

fos = None


def login(data):
    host = data['host']
    username = data['username']
    password = data['password']

    fos.debug('on')
    if 'https' in data and not data['https']:
        fos.https('off')
    else:
        fos.https('on')

    fos.login(host, username, password)


def filter_router_ospf_data(json):
    option_list = ['abr-type', 'area', 'auto-cost-ref-bandwidth',
                   'bfd', 'database-overflow', 'database-overflow-max-lsas',
                   'database-overflow-time-to-recover', 'default-information-metric', 'default-information-metric-type',
                   'default-information-originate', 'default-information-route-map', 'default-metric',
                   'distance', 'distance-external', 'distance-inter-area',
                   'distance-intra-area', 'distribute-list', 'distribute-list-in',
                   'distribute-route-map-in', 'log-neighbour-changes', 'neighbor',
                   'network', 'ospf-interface', 'passive-interface',
                   'redistribute', 'restart-mode', 'restart-period',
                   'rfc1583-compatible', 'router-id', 'spf-timers',
                   'summary-address']
    dictionary = {}

    for attribute in option_list:
        if attribute in json and json[attribute] is not None:
            dictionary[attribute] = json[attribute]

    return dictionary


def flatten_multilists_attributes(data):
    multilist_attrs = []

    for attr in multilist_attrs:
        try:
            path = "data['" + "']['".join(elem for elem in attr) + "']"
            current_val = eval(path)
            flattened_val = ' '.join(elem for elem in current_val)
            exec(path + '= flattened_val')
        except BaseException:
            pass

    return data


def router_ospf(data, fos):
    vdom = data['vdom']
    router_ospf_data = data['router_ospf']
    flattened_data = flatten_multilists_attributes(router_ospf_data)
    filtered_data = filter_router_ospf_data(flattened_data)
    return fos.set('router',
                   'ospf',
                   data=filtered_data,
                   vdom=vdom)


def fortios_router(data, fos):
    login(data)

    if data['router_ospf']:
        resp = router_ospf(data, fos)

    fos.logout()
    return not resp['status'] == "success", resp['status'] == "success", resp


def main():
    fields = {
        "host": {"required": True, "type": "str"},
        "username": {"required": True, "type": "str"},
        "password": {"required": False, "type": "str", "no_log": True},
        "vdom": {"required": False, "type": "str", "default": "root"},
        "https": {"required": False, "type": "bool", "default": True},
        "router_ospf": {
            "required": False, "type": "dict",
            "options": {
                "abr-type": {"required": False, "type": "str",
                             "choices": ["cisco", "ibm", "shortcut",
                                         "standard"]},
                "area": {"required": False, "type": "list",
                         "options": {
                             "authentication": {"required": False, "type": "str",
                                                "choices": ["none", "text", "md5"]},
                             "default-cost": {"required": False, "type": "int"},
                             "filter-list": {"required": False, "type": "list",
                                             "options": {
                                                 "direction": {"required": False, "type": "str",
                                                               "choices": ["in", "out"]},
                                                 "id": {"required": True, "type": "int"},
                                                 "list": {"required": False, "type": "str"}
                                             }},
                             "id": {"required": True, "type": "str"},
                             "nssa-default-information-originate": {"required": False, "type": "str",
                                                                    "choices": ["enable", "always", "disable"]},
                             "nssa-default-information-originate-metric": {"required": False, "type": "int"},
                             "nssa-default-information-originate-metric-type": {"required": False, "type": "str",
                                                                                "choices": ["1", "2"]},
                             "nssa-redistribution": {"required": False, "type": "str",
                                                     "choices": ["enable", "disable"]},
                             "nssa-translator-role": {"required": False, "type": "str",
                                                      "choices": ["candidate", "never", "always"]},
                             "range": {"required": False, "type": "list",
                                       "options": {
                                           "advertise": {"required": False, "type": "str",
                                                         "choices": ["disable", "enable"]},
                                           "id": {"required": True, "type": "int"},
                                           "prefix": {"required": False, "type": "str"},
                                           "substitute": {"required": False, "type": "str"},
                                           "substitute-status": {"required": False, "type": "str",
                                                                 "choices": ["enable", "disable"]}
                                       }},
                             "shortcut": {"required": False, "type": "str",
                                          "choices": ["disable", "enable", "default"]},
                             "stub-type": {"required": False, "type": "str",
                                           "choices": ["no-summary", "summary"]},
                             "type": {"required": False, "type": "str",
                                      "choices": ["regular", "nssa", "stub"]},
                             "virtual-link": {"required": False, "type": "list",
                                              "options": {
                                                  "authentication": {"required": False, "type": "str",
                                                                     "choices": ["none", "text", "md5"]},
                                                  "authentication-key": {"required": False, "type": "str"},
                                                  "dead-interval": {"required": False, "type": "int"},
                                                  "hello-interval": {"required": False, "type": "int"},
                                                  "md5-key": {"required": False, "type": "str"},
                                                  "name": {"required": True, "type": "str"},
                                                  "peer": {"required": False, "type": "str"},
                                                  "retransmit-interval": {"required": False, "type": "int"},
                                                  "transmit-delay": {"required": False, "type": "int"}
                                              }}
                         }},
                "auto-cost-ref-bandwidth": {"required": False, "type": "int"},
                "bfd": {"required": False, "type": "str",
                        "choices": ["enable", "disable"]},
                "database-overflow": {"required": False, "type": "str",
                                      "choices": ["enable", "disable"]},
                "database-overflow-max-lsas": {"required": False, "type": "int"},
                "database-overflow-time-to-recover": {"required": False, "type": "int"},
                "default-information-metric": {"required": False, "type": "int"},
                "default-information-metric-type": {"required": False, "type": "str",
                                                    "choices": ["1", "2"]},
                "default-information-originate": {"required": False, "type": "str",
                                                  "choices": ["enable", "always", "disable"]},
                "default-information-route-map": {"required": False, "type": "str"},
                "default-metric": {"required": False, "type": "int"},
                "distance": {"required": False, "type": "int"},
                "distance-external": {"required": False, "type": "int"},
                "distance-inter-area": {"required": False, "type": "int"},
                "distance-intra-area": {"required": False, "type": "int"},
                "distribute-list": {"required": False, "type": "list",
                                    "options": {
                                        "access-list": {"required": False, "type": "str"},
                                        "id": {"required": True, "type": "int"},
                                        "protocol": {"required": False, "type": "str",
                                                     "choices": ["connected", "static", "rip"]}
                                    }},
                "distribute-list-in": {"required": False, "type": "str"},
                "distribute-route-map-in": {"required": False, "type": "str"},
                "log-neighbour-changes": {"required": False, "type": "str",
                                          "choices": ["enable", "disable"]},
                "neighbor": {"required": False, "type": "list",
                             "options": {
                                 "cost": {"required": False, "type": "int"},
                                 "id": {"required": True, "type": "int"},
                                 "ip": {"required": False, "type": "str"},
                                 "poll-interval": {"required": False, "type": "int"},
                                 "priority": {"required": False, "type": "int"}
                             }},
                "network": {"required": False, "type": "list",
                            "options": {
                                "area": {"required": False, "type": "str"},
                                "id": {"required": True, "type": "int"},
                                "prefix": {"required": False, "type": "str"}
                            }},
                "ospf-interface": {"required": False, "type": "list",
                                   "options": {
                                       "authentication": {"required": False, "type": "str",
                                                          "choices": ["none", "text", "md5"]},
                                       "authentication-key": {"required": False, "type": "str"},
                                       "bfd": {"required": False, "type": "str",
                                               "choices": ["global", "enable", "disable"]},
                                       "cost": {"required": False, "type": "int"},
                                       "database-filter-out": {"required": False, "type": "str",
                                                               "choices": ["enable", "disable"]},
                                       "dead-interval": {"required": False, "type": "int"},
                                       "hello-interval": {"required": False, "type": "int"},
                                       "hello-multiplier": {"required": False, "type": "int"},
                                       "interface": {"required": False, "type": "str"},
                                       "ip": {"required": False, "type": "str"},
                                       "md5-key": {"required": False, "type": "str"},
                                       "mtu": {"required": False, "type": "int"},
                                       "mtu-ignore": {"required": False, "type": "str",
                                                      "choices": ["enable", "disable"]},
                                       "name": {"required": True, "type": "str"},
                                       "network-type": {"required": False, "type": "str",
                                                        "choices": ["broadcast", "non-broadcast", "point-to-point",
                                                                    "point-to-multipoint", "point-to-multipoint-non-broadcast"]},
                                       "prefix-length": {"required": False, "type": "int"},
                                       "priority": {"required": False, "type": "int"},
                                       "resync-timeout": {"required": False, "type": "int"},
                                       "retransmit-interval": {"required": False, "type": "int"},
                                       "status": {"required": False, "type": "str",
                                                  "choices": ["disable", "enable"]},
                                       "transmit-delay": {"required": False, "type": "int"}
                                   }},
                "passive-interface": {"required": False, "type": "list",
                                      "options": {
                                          "name": {"required": True, "type": "str"}
                                      }},
                "redistribute": {"required": False, "type": "list",
                                 "options": {
                                     "metric": {"required": False, "type": "int"},
                                     "metric-type": {"required": False, "type": "str",
                                                     "choices": ["1", "2"]},
                                     "name": {"required": True, "type": "str"},
                                     "routemap": {"required": False, "type": "str"},
                                     "status": {"required": False, "type": "str",
                                                "choices": ["enable", "disable"]},
                                     "tag": {"required": False, "type": "int"}
                                 }},
                "restart-mode": {"required": False, "type": "str",
                                 "choices": ["none", "lls", "graceful-restart"]},
                "restart-period": {"required": False, "type": "int"},
                "rfc1583-compatible": {"required": False, "type": "str",
                                       "choices": ["enable", "disable"]},
                "router-id": {"required": False, "type": "str"},
                "spf-timers": {"required": False, "type": "str"},
                "summary-address": {"required": False, "type": "list",
                                    "options": {
                                        "advertise": {"required": False, "type": "str",
                                                      "choices": ["disable", "enable"]},
                                        "id": {"required": True, "type": "int"},
                                        "prefix": {"required": False, "type": "str"},
                                        "tag": {"required": False, "type": "int"}
                                    }}

            }
        }
    }

    module = AnsibleModule(argument_spec=fields,
                           supports_check_mode=False)
    try:
        from fortiosapi import FortiOSAPI
    except ImportError:
        module.fail_json(msg="fortiosapi module is required")

    global fos
    fos = FortiOSAPI()

    is_error, has_changed, result = fortios_router(module.params, fos)

    if not is_error:
        module.exit_json(changed=has_changed, meta=result)
    else:
        module.fail_json(msg="Error in repo", meta=result)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
#
# Copyright (c) 2017 F5 Networks Inc.
# GNU General Public License v3.0 (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import os
import json
import pytest
import sys

if sys.version_info < (2, 7):
    pytestmark = pytest.mark.skip("F5 Ansible modules require Python >= 2.7")

from ansible.module_utils.basic import AnsibleModule

try:
    from library.modules.bigip_sys_db import Parameters
    from library.modules.bigip_sys_db import ModuleManager
    from library.modules.bigip_sys_db import ArgumentSpec

    # In Ansible 2.8, Ansible changed import paths.
    from test.units.compat import unittest
    from test.units.compat.mock import Mock
    from test.units.compat.mock import patch

    from test.units.modules.utils import set_module_args
except ImportError:
    from ansible.modules.network.f5.bigip_sys_db import Parameters
    from ansible.modules.network.f5.bigip_sys_db import ModuleManager
    from ansible.modules.network.f5.bigip_sys_db import ArgumentSpec

    # Ansible 2.8 imports
    from units.compat import unittest
    from units.compat.mock import Mock
    from units.compat.mock import patch

    from units.modules.utils import set_module_args


fixture_path = os.path.join(os.path.dirname(__file__), 'fixtures')
fixture_data = {}


def load_fixture(name):
    path = os.path.join(fixture_path, name)

    if path in fixture_data:
        return fixture_data[path]

    with open(path) as f:
        data = f.read()

    try:
        data = json.loads(data)
    except Exception:
        pass

    fixture_data[path] = data
    return data


class TestParameters(unittest.TestCase):
    def test_module_parameters(self):
        args = dict(
            key='foo',
            value='bar',
        )
        p = Parameters(params=args)
        assert p.key == 'foo'
        assert p.value == 'bar'

    def test_api_parameters(self):
        args = dict(
            key='foo',
            value='bar',
            defaultValue='baz',

        )
        p = Parameters(params=args)
        assert p.key == 'foo'
        assert p.value == 'bar'


class TestManager(unittest.TestCase):

    def setUp(self):
        self.spec = ArgumentSpec()

    def test_create_blackhole(self, *args):
        set_module_args(dict(
            key='provision.cpu.afm',
            value='1',
            state='present',
            provider=dict(
                server='localhost',
                password='password',
                user='admin'
            )
        ))

        # Configure the parameters that would be returned by querying the
        # remote device
        current = Parameters(
            dict(
                kind="tm:sys:db:dbstate",
                name="provision.cpu.afm",
                fullPath="provision.cpu.afm",
                generation=1,
                selfLink="https://localhost/mgmt/tm/sys/db/provision.cpu.afm?ver=11.6.1",
                defaultValue="0",
                scfConfig="false",
                value="0",
                valueRange="integer min:0 max:100"
            )
        )

        module = AnsibleModule(
            argument_spec=self.spec.argument_spec,
            supports_check_mode=self.spec.supports_check_mode
        )
        mm = ModuleManager(module=module)

        # Override methods to force specific logic in the module to happen
        mm.exists = Mock(return_value=False)
        mm.read_current_from_device = Mock(return_value=current)
        mm.update_on_device = Mock(return_value=True)

        results = mm.exec_module()
        assert results['changed'] is True



# -*- coding: utf-8 -*-
#
# Copyright: (c) 2017, F5 Networks Inc.
# GNU General Public License v3.0 (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import os
import json
import pytest
import sys

if sys.version_info < (2, 7):
    pytestmark = pytest.mark.skip("F5 Ansible modules require Python >= 2.7")

from ansible.module_utils.basic import AnsibleModule

try:
    from library.modules.bigip_firewall_port_list import ApiParameters
    from library.modules.bigip_firewall_port_list import ModuleParameters
    from library.modules.bigip_firewall_port_list import ModuleManager
    from library.modules.bigip_firewall_port_list import ArgumentSpec

    # In Ansible 2.8, Ansible changed import paths.
    from test.units.compat import unittest
    from test.units.compat.mock import Mock
    from test.units.compat.mock import patch

    from test.units.modules.utils import set_module_args
except ImportError:
    from ansible.modules.network.f5.bigip_firewall_port_list import ApiParameters
    from ansible.modules.network.f5.bigip_firewall_port_list import ModuleParameters
    from ansible.modules.network.f5.bigip_firewall_port_list import ModuleManager
    from ansible.modules.network.f5.bigip_firewall_port_list import ArgumentSpec

    # Ansible 2.8 imports
    from units.compat import unittest
    from units.compat.mock import Mock
    from units.compat.mock import patch

    from units.modules.utils import set_module_args


fixture_path = os.path.join(os.path.dirname(__file__), 'fixtures')
fixture_data = {}


def load_fixture(name):
    path = os.path.join(fixture_path, name)

    if path in fixture_data:
        return fixture_data[path]

    with open(path) as f:
        data = f.read()

    try:
        data = json.loads(data)
    except Exception:
        pass

    fixture_data[path] = data
    return data


class TestParameters(unittest.TestCase):
    def test_module_parameters(self):
        args = dict(
            name='foo',
            description='this is a description',
            ports=[1, 2, 3, 4],
            port_ranges=['10-20', '30-40', '50-60'],
            port_lists=['/Common/foo', 'foo']
        )

        p = ModuleParameters(params=args)
        assert p.name == 'foo'
        assert p.description == 'this is a description'
        assert len(p.ports) == 4
        assert len(p.port_ranges) == 3
        assert len(p.port_lists) == 2

    def test_api_parameters(self):
        args = load_fixture('load_security_port_list_1.json')

        p = ApiParameters(params=args)
        assert len(p.ports) == 4
        assert len(p.port_ranges) == 3
        assert len(p.port_lists) == 1
        assert sorted(p.ports) == [1, 2, 3, 4]
        assert sorted(p.port_ranges) == ['10-20', '30-40', '50-60']
        assert p.port_lists[0] == '/Common/_sys_self_allow_tcp_defaults'


class TestManager(unittest.TestCase):

    def setUp(self):
        self.spec = ArgumentSpec()

        try:
            self.p1 = patch('library.modules.bigip_firewall_port_list.module_provisioned')
            self.m1 = self.p1.start()
            self.m1.return_value = True
        except Exception:
            self.p1 = patch('ansible.modules.network.f5.bigip_firewall_port_list.module_provisioned')
            self.m1 = self.p1.start()
            self.m1.return_value = True

    def tearDown(self):
        self.p1.stop()

    def test_create(self, *args):
        set_module_args(dict(
            name='foo',
            description='this is a description',
            ports=[1, 2, 3, 4],
            port_ranges=['10-20', '30-40', '50-60'],
            port_lists=['/Common/foo', 'foo'],
            provider=dict(
                server='localhost',
                password='password',
                user='admin'
            )
        ))

        module = AnsibleModule(
            argument_spec=self.spec.argument_spec,
            supports_check_mode=self.spec.supports_check_mode
        )
        mm = ModuleManager(module=module)

        # Override methods to force specific logic in the module to happen
        mm.exists = Mock(side_effect=[False, True])
        mm.create_on_device = Mock(return_value=True)
        mm.module_provisioned = Mock(return_value=True)

        results = mm.exec_module()

        assert results['changed'] is True
        assert 'ports' in results
        assert 'port_lists' in results
        assert 'port_ranges' in results
        assert len(results['ports']) == 4
        assert len(results['port_ranges']) == 3
        assert len(results['port_lists']) == 2
        assert results['description'] == 'this is a description'



# (c) 2017, Adrian Likins <alikins@redhat.com>
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.

# Make coding more python3-ish
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

from units.compat import unittest
from units.compat.mock import patch
from units.mock.vault_helper import TextVaultSecret

from ansible import errors
from ansible.cli.vault import VaultCLI


# TODO: make these tests assert something, likely by verifing
#       mock calls


class TestVaultCli(unittest.TestCase):
    def setUp(self):
        self.tty_patcher = patch('ansible.cli.sys.stdin.isatty', return_value=False)
        self.mock_isatty = self.tty_patcher.start()

    def tearDown(self):
        self.tty_patcher.stop()

    def test_parse_empty(self):
        cli = VaultCLI([])
        self.assertRaisesRegexp(errors.AnsibleOptionsError,
                                '.*Missing required action.*',
                                cli.parse)

    # FIXME: something weird seems to be afoot when parsing actions
    # cli = VaultCLI(args=['view', '/dev/null/foo', 'mysecret3'])
    # will skip '/dev/null/foo'. something in cli.CLI.set_action() ?
    #   maybe we self.args gets modified in a loop?
    def test_parse_view_file(self):
        cli = VaultCLI(args=['ansible-vault', 'view', '/dev/null/foo'])
        cli.parse()

    @patch('ansible.cli.vault.VaultCLI.setup_vault_secrets')
    def test_view_missing_file_no_secret(self, mock_setup_vault_secrets):
        mock_setup_vault_secrets.return_value = []
        cli = VaultCLI(args=['ansible-vault', 'view', '/dev/null/foo'])
        cli.parse()
        self.assertRaisesRegexp(errors.AnsibleOptionsError,
                                "A vault password is required to use Ansible's Vault",
                                cli.run)

    @patch('ansible.cli.vault.VaultCLI.setup_vault_secrets')
    def test_encrypt_missing_file_no_secret(self, mock_setup_vault_secrets):
        mock_setup_vault_secrets.return_value = []
        cli = VaultCLI(args=['ansible-vault', 'encrypt', '/dev/null/foo'])
        cli.parse()
        self.assertRaisesRegexp(errors.AnsibleOptionsError,
                                "A vault password is required to use Ansible's Vault",
                                cli.run)

    @patch('ansible.cli.vault.VaultCLI.setup_vault_secrets')
    @patch('ansible.cli.vault.VaultEditor')
    def test_encrypt(self, mock_vault_editor, mock_setup_vault_secrets):
        mock_setup_vault_secrets.return_value = [('default', TextVaultSecret('password'))]
        cli = VaultCLI(args=['ansible-vault', 'encrypt', '/dev/null/foo'])
        cli.parse()
        cli.run()

    @patch('ansible.cli.vault.VaultCLI.setup_vault_secrets')
    @patch('ansible.cli.vault.VaultEditor')
    def test_encrypt_string(self, mock_vault_editor, mock_setup_vault_secrets):
        mock_setup_vault_secrets.return_value = [('default', TextVaultSecret('password'))]
        cli = VaultCLI(args=['ansible-vault', 'encrypt_string',
                             'some string to encrypt'])
        cli.parse()
        cli.run()

    @patch('ansible.cli.vault.VaultCLI.setup_vault_secrets')
    @patch('ansible.cli.vault.VaultEditor')
    @patch('ansible.cli.vault.display.prompt', return_value='a_prompt')
    def test_encrypt_string_prompt(self, mock_display, mock_vault_editor, mock_setup_vault_secrets):
        mock_setup_vault_secrets.return_value = [('default', TextVaultSecret('password'))]
        cli = VaultCLI(args=['ansible-vault',
                             'encrypt_string',
                             '--prompt',
                             'some string to encrypt'])
        cli.parse()
        cli.run()

    @patch('ansible.cli.vault.VaultCLI.setup_vault_secrets')
    @patch('ansible.cli.vault.VaultEditor')
    @patch('ansible.cli.vault.sys.stdin.read', return_value='This is data from stdin')
    def test_encrypt_string_stdin(self, mock_stdin_read, mock_vault_editor, mock_setup_vault_secrets):
        mock_setup_vault_secrets.return_value = [('default', TextVaultSecret('password'))]
        cli = VaultCLI(args=['ansible-vault',
                             'encrypt_string',
                             '--stdin-name',
                             'the_var_from_stdin',
                             '-'])
        cli.parse()
        cli.run()

    @patch('ansible.cli.vault.VaultCLI.setup_vault_secrets')
    @patch('ansible.cli.vault.VaultEditor')
    def test_encrypt_string_names(self, mock_vault_editor, mock_setup_vault_secrets):
        mock_setup_vault_secrets.return_value = [('default', TextVaultSecret('password'))]
        cli = VaultCLI(args=['ansible-vault', 'encrypt_string',
                             '--name', 'foo1',
                             '--name', 'foo2',
                             'some string to encrypt'])
        cli.parse()
        cli.run()

    @patch('ansible.cli.vault.VaultCLI.setup_vault_secrets')
    @patch('ansible.cli.vault.VaultEditor')
    def test_encrypt_string_more_args_than_names(self, mock_vault_editor, mock_setup_vault_secrets):
        mock_setup_vault_secrets.return_value = [('default', TextVaultSecret('password'))]
        cli = VaultCLI(args=['ansible-vault', 'encrypt_string',
                             '--name', 'foo1',
                             'some string to encrypt',
                             'other strings',
                             'a few more string args'])
        cli.parse()
        cli.run()

    @patch('ansible.cli.vault.VaultCLI.setup_vault_secrets')
    @patch('ansible.cli.vault.VaultEditor')
    def test_create(self, mock_vault_editor, mock_setup_vault_secrets):
        mock_setup_vault_secrets.return_value = [('default', TextVaultSecret('password'))]
        cli = VaultCLI(args=['ansible-vault', 'create', '/dev/null/foo'])
        cli.parse()
        cli.run()

    @patch('ansible.cli.vault.VaultCLI.setup_vault_secrets')
    @patch('ansible.cli.vault.VaultEditor')
    def test_edit(self, mock_vault_editor, mock_setup_vault_secrets):
        mock_setup_vault_secrets.return_value = [('default', TextVaultSecret('password'))]
        cli = VaultCLI(args=['ansible-vault', 'edit', '/dev/null/foo'])
        cli.parse()
        cli.run()

    @patch('ansible.cli.vault.VaultCLI.setup_vault_secrets')
    @patch('ansible.cli.vault.VaultEditor')
    def test_decrypt(self, mock_vault_editor, mock_setup_vault_secrets):
        mock_setup_vault_secrets.return_value = [('default', TextVaultSecret('password'))]
        cli = VaultCLI(args=['ansible-vault', 'decrypt', '/dev/null/foo'])
        cli.parse()
        cli.run()

    @patch('ansible.cli.vault.VaultCLI.setup_vault_secrets')
    @patch('ansible.cli.vault.VaultEditor')
    def test_view(self, mock_vault_editor, mock_setup_vault_secrets):
        mock_setup_vault_secrets.return_value = [('default', TextVaultSecret('password'))]
        cli = VaultCLI(args=['ansible-vault', 'view', '/dev/null/foo'])
        cli.parse()
        cli.run()

    @patch('ansible.cli.vault.VaultCLI.setup_vault_secrets')
    @patch('ansible.cli.vault.VaultEditor')
    def test_rekey(self, mock_vault_editor, mock_setup_vault_secrets):
        mock_setup_vault_secrets.return_value = [('default', TextVaultSecret('password'))]
        cli = VaultCLI(args=['ansible-vault', 'rekey', '/dev/null/foo'])
        cli.parse()
        cli.run()



"""Ansible integration test infrastructure."""

from __future__ import absolute_import, print_function

import contextlib
import json
import os
import shutil
import stat
import tempfile

from lib.target import (
    analyze_integration_target_dependencies,
    walk_integration_targets,
)

from lib.config import (
    NetworkIntegrationConfig,
    PosixIntegrationConfig,
    WindowsIntegrationConfig,
)

from lib.util import (
    ApplicationError,
    display,
    make_dirs,
    named_temporary_file,
    COVERAGE_CONFIG_PATH,
    COVERAGE_OUTPUT_PATH,
    MODE_DIRECTORY,
    MODE_DIRECTORY_WRITE,
    MODE_FILE,
)

from lib.cache import (
    CommonCache,
)

from lib.cloud import (
    CloudEnvironmentConfig,
)


def setup_common_temp_dir(args, path):
    """
    :type args: IntegrationConfig
    :type path: str
    """
    if args.explain:
        return

    os.mkdir(path)
    os.chmod(path, MODE_DIRECTORY)

    coverage_config_path = os.path.join(path, COVERAGE_CONFIG_PATH)

    shutil.copy(COVERAGE_CONFIG_PATH, coverage_config_path)
    os.chmod(coverage_config_path, MODE_FILE)

    coverage_output_path = os.path.join(path, COVERAGE_OUTPUT_PATH)

    os.mkdir(coverage_output_path)
    os.chmod(coverage_output_path, MODE_DIRECTORY_WRITE)


def generate_dependency_map(integration_targets):
    """
    :type integration_targets: list[IntegrationTarget]
    :rtype: dict[str, set[IntegrationTarget]]
    """
    targets_dict = dict((target.name, target) for target in integration_targets)
    target_dependencies = analyze_integration_target_dependencies(integration_targets)
    dependency_map = {}

    invalid_targets = set()

    for dependency, dependents in target_dependencies.items():
        dependency_target = targets_dict.get(dependency)

        if not dependency_target:
            invalid_targets.add(dependency)
            continue

        for dependent in dependents:
            if dependent not in dependency_map:
                dependency_map[dependent] = set()

            dependency_map[dependent].add(dependency_target)

    if invalid_targets:
        raise ApplicationError('Non-existent target dependencies: %s' % ', '.join(sorted(invalid_targets)))

    return dependency_map


def get_files_needed(target_dependencies):
    """
    :type target_dependencies: list[IntegrationTarget]
    :rtype: list[str]
    """
    files_needed = []

    for target_dependency in target_dependencies:
        files_needed += target_dependency.needs_file

    files_needed = sorted(set(files_needed))

    invalid_paths = [path for path in files_needed if not os.path.isfile(path)]

    if invalid_paths:
        raise ApplicationError('Invalid "needs/file/*" aliases:\n%s' % '\n'.join(invalid_paths))

    return files_needed


@contextlib.contextmanager
def integration_test_environment(args, target, inventory_path):
    """
    :type args: IntegrationConfig
    :type target: IntegrationTarget
    :type inventory_path: str
    """
    vars_file = 'integration_config.yml'

    if args.no_temp_workdir or 'no/temp_workdir/' in target.aliases:
        display.warning('Disabling the temp work dir is a temporary debugging feature that may be removed in the future without notice.')

        integration_dir = os.path.abspath('test/integration')
        inventory_path = os.path.abspath(inventory_path)
        ansible_config = os.path.join(integration_dir, '%s.cfg' % args.command)
        vars_file = os.path.join(integration_dir, vars_file)

        yield IntegrationEnvironment(integration_dir, inventory_path, ansible_config, vars_file)
        return

    root_temp_dir = os.path.expanduser('~/.ansible/test/tmp')

    prefix = '%s-' % target.name
    suffix = u'-\u00c5\u00d1\u015a\u00cc\u03b2\u0141\u00c8'

    if args.no_temp_unicode or 'no/temp_unicode/' in target.aliases:
        display.warning('Disabling unicode in the temp work dir is a temporary debugging feature that may be removed in the future without notice.')
        suffix = '-ansible'

    if isinstance('', bytes):
        suffix = suffix.encode('utf-8')

    if args.explain:
        temp_dir = os.path.join(root_temp_dir, '%stemp%s' % (prefix, suffix))
    else:
        make_dirs(root_temp_dir)
        temp_dir = tempfile.mkdtemp(prefix=prefix, suffix=suffix, dir=root_temp_dir)

    try:
        display.info('Preparing temporary directory: %s' % temp_dir, verbosity=2)

        inventory_names = {
            PosixIntegrationConfig: 'inventory',
            WindowsIntegrationConfig: 'inventory.winrm',
            NetworkIntegrationConfig: 'inventory.networking',
        }

        inventory_name = inventory_names[type(args)]

        cache = IntegrationCache(args)

        target_dependencies = sorted([target] + list(cache.dependency_map.get(target.name, set())))

        files_needed = get_files_needed(target_dependencies)

        integration_dir = os.path.join(temp_dir, 'test/integration')
        ansible_config = os.path.join(integration_dir, '%s.cfg' % args.command)

        file_copies = [
            ('test/integration/%s.cfg' % args.command, ansible_config),
            ('test/integration/integration_config.yml', os.path.join(integration_dir, vars_file)),
            (inventory_path, os.path.join(integration_dir, inventory_name)),
        ]

        file_copies += [(path, os.path.join(temp_dir, path)) for path in files_needed]

        directory_copies = [
            (os.path.join('test/integration/targets', target.name), os.path.join(integration_dir, 'targets', target.name)) for target in target_dependencies
        ]

        inventory_dir = os.path.dirname(inventory_path)

        host_vars_dir = os.path.join(inventory_dir, 'host_vars')
        group_vars_dir = os.path.join(inventory_dir, 'group_vars')

        if os.path.isdir(host_vars_dir):
            directory_copies.append((host_vars_dir, os.path.join(integration_dir, os.path.basename(host_vars_dir))))

        if os.path.isdir(group_vars_dir):
            directory_copies.append((group_vars_dir, os.path.join(integration_dir, os.path.basename(group_vars_dir))))

        directory_copies = sorted(set(directory_copies))
        file_copies = sorted(set(file_copies))

        if not args.explain:
            make_dirs(integration_dir)

        for dir_src, dir_dst in directory_copies:
            display.info('Copying %s/ to %s/' % (dir_src, dir_dst), verbosity=2)

            if not args.explain:
                shutil.copytree(dir_src, dir_dst, symlinks=True)

        for file_src, file_dst in file_copies:
            display.info('Copying %s to %s' % (file_src, file_dst), verbosity=2)

            if not args.explain:
                make_dirs(os.path.dirname(file_dst))
                shutil.copy2(file_src, file_dst)

        inventory_path = os.path.join(integration_dir, inventory_name)
        vars_file = os.path.join(integration_dir, vars_file)

        yield IntegrationEnvironment(integration_dir, inventory_path, ansible_config, vars_file)
    finally:
        if not args.explain:
            shutil.rmtree(temp_dir)


@contextlib.contextmanager
def integration_test_config_file(args, env_config, integration_dir):
    """
    :type args: IntegrationConfig
    :type env_config: CloudEnvironmentConfig
    :type integration_dir: str
    """
    if not env_config:
        yield None
        return

    config_vars = (env_config.ansible_vars or {}).copy()

    config_vars.update(dict(
        ansible_test=dict(
            environment=env_config.env_vars,
            module_defaults=env_config.module_defaults,
        )
    ))

    config_file = json.dumps(config_vars, indent=4, sort_keys=True)

    with named_temporary_file(args, 'config-file-', '.json', integration_dir, config_file) as path:
        filename = os.path.relpath(path, integration_dir)

        display.info('>>> Config File: %s\n%s' % (filename, config_file), verbosity=3)

        yield path


class IntegrationEnvironment(object):
    """Details about the integration environment."""
    def __init__(self, integration_dir, inventory_path, ansible_config, vars_file):
        self.integration_dir = integration_dir
        self.inventory_path = inventory_path
        self.ansible_config = ansible_config
        self.vars_file = vars_file


class IntegrationCache(CommonCache):
    """Integration cache."""
    @property
    def integration_targets(self):
        """
        :rtype: list[IntegrationTarget]
        """
        return self.get('integration_targets', lambda: list(walk_integration_targets()))

    @property
    def dependency_map(self):
        """
        :rtype: dict[str, set[IntegrationTarget]]
        """
        return self.get('dependency_map', lambda: generate_dependency_map(self.integration_targets))



# (c) 2016 Red Hat Inc.
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.

# Make coding more python3-ish
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import json

from units.compat.mock import patch
from ansible.modules.network.nxos import nxos_vlan
from .nxos_module import TestNxosModule, load_fixture, set_module_args


class TestNxosVlanModule(TestNxosModule):

    module = nxos_vlan

    def setUp(self):
        super(TestNxosVlanModule, self).setUp()

        self.mock_run_commands = patch('ansible.modules.network.nxos.nxos_vlan.run_commands')
        self.run_commands = self.mock_run_commands.start()

        self.mock_load_config = patch('ansible.modules.network.nxos.nxos_vlan.load_config')
        self.load_config = self.mock_load_config.start()

        self.mock_get_config = patch('ansible.modules.network.nxos.nxos_vlan.get_config')
        self.get_config = self.mock_get_config.start()

        self.mock_get_capabilities = patch('ansible.modules.network.nxos.nxos_vlan.get_capabilities')
        self.get_capabilities = self.mock_get_capabilities.start()
        self.get_capabilities.return_value = {'device_info': {'network_os_platform': 'N9K-9000v'}, 'network_api': 'cliconf'}

    def tearDown(self):
        super(TestNxosVlanModule, self).tearDown()
        self.mock_run_commands.stop()
        self.mock_load_config.stop()
        self.mock_get_config.stop()
        self.mock_get_capabilities.stop()

    def load_fixtures(self, commands=None, device=''):
        def load_from_file(*args, **kwargs):
            module, commands = args
            output = list()

            for item in commands:
                try:
                    obj = json.loads(item)
                    command = obj['command']
                except ValueError:
                    command = item
                filename = '%s.txt' % str(command).split(' | ')[0].replace(' ', '_')
                output.append(load_fixture('nxos_vlan', filename))
            return output

        def agg_load_from_file(*args, **kwargs):
            """Load vlan output for aggregate/purge tests"""
            return([load_fixture('nxos_vlan', 'agg_show_vlan_brief.txt')])

        if '_agg_' in self._testMethodName:
            self.run_commands.side_effect = agg_load_from_file
        else:
            self.run_commands.side_effect = load_from_file

        self.load_config.return_value = None
        self.get_config.return_value = load_fixture('nxos_vlan', 'config.cfg')

    def test_nxos_vlan_agg_1(self):
        # Aggregate: vlan 4/5 exist -> Add 6
        set_module_args(dict(aggregate=[
            {'name': '_5_', 'vlan_id': 5},
            {'name': '_6_', 'vlan_id': 6}
        ]))
        self.execute_module(changed=True, commands=[
            'vlan 6',
            'name _6_',
            'state active',
            'no shutdown',
            'exit'
        ])

    def test_nxos_vlan_agg_2(self):
        # Aggregate: vlan 4/5 exist -> Add none (idempotence)
        set_module_args(dict(aggregate=[
            {'name': '_5_', 'vlan_id': 5},
            {'name': '_4_', 'vlan_id': 4}
        ]))
        self.execute_module(changed=False)

    def test_nxos_vlan_agg_3(self):
        # Aggregate/Purge: vlan 4/5 exist -> Add 6, Purge 4
        set_module_args(dict(aggregate=[
            {'name': '_5_', 'vlan_id': 5},
            {'name': '_6_', 'vlan_id': 6}
        ], purge=True))
        self.execute_module(changed=True, commands=[
            'vlan 6',
            'name _6_',
            'state active',
            'no shutdown',
            'exit',
            'no vlan 4'
        ])

    def test_nxos_vlan_agg_4(self):
        # Aggregate/Purge: vlan 4/5 exist -> Purge None (idempotence)
        set_module_args(dict(aggregate=[
            {'name': '_5_', 'vlan_id': 5},
            {'name': '_4_', 'vlan_id': 4}
        ]))
        self.execute_module(changed=False)

    def test_nxos_vlan_agg_5(self):
        # Purge with Single Vlan: vlan 4/5 exist -> Add 6, Purge 4/5
        set_module_args(dict(vlan_id=6, name='_6_', purge=True))
        self.execute_module(changed=True, commands=[
            'vlan 6',
            'name _6_',
            'state active',
            'no shutdown',
            'exit',
            'no vlan 4',
            'no vlan 5'
        ])

    def test_nxos_vlan_agg_6(self):
        # Purge All: vlan 4/5 exist -> Purge 4/5
        set_module_args(dict(vlan_id=1, purge=True))
        self.execute_module(changed=True, commands=[
            'no vlan 4',
            'no vlan 5'
        ])

    def test_nxos_vlan_range(self):
        set_module_args(dict(vlan_range='6-10'))
        self.execute_module(changed=True, commands=['vlan 6', 'vlan 7', 'vlan 8', 'vlan 9', 'vlan 10'])

    def test_nxos_vlan_range_absent(self):
        set_module_args(dict(vlan_range='1-5', state='absent'))
        result = self.execute_module(changed=True)
        self.assertEqual(result['commands'], ['no vlan 1'])

    def test_nxos_vlan_id(self):
        set_module_args(dict(vlan_id='15', state='present'))
        result = self.execute_module(changed=True)
        self.assertEqual(result['commands'], ['vlan 15', 'state active', 'no shutdown', 'exit'])

    def test_nxos_vlan_id_absent(self):
        set_module_args(dict(vlan_id='1', state='absent'))
        result = self.execute_module(changed=True)
        self.assertEqual(result['commands'], ['no vlan 1'])

    def test_nxos_vlan_named_vlan(self):
        set_module_args(dict(vlan_id='15', name='WEB'))
        result = self.execute_module(changed=True)
        self.assertEqual(result['commands'], ['vlan 15', 'name WEB', 'state active', 'no shutdown', 'exit'])

    def test_nxos_vlan_shut_down(self):
        set_module_args(dict(vlan_id='1', admin_state='down'))
        result = self.execute_module(changed=True)
        self.assertEqual(result['commands'], ['vlan 1', 'shutdown', 'exit'])

    def test_nxos_vlan_no_change(self):
        set_module_args(dict(vlan_id='1', name='default', vlan_state='active', admin_state='up'))
        result = self.execute_module(changed=False)
        self.assertEqual(result['commands'], [])
